#! /usr/bin/env python
#    Visualization for Bifurcation Manifolds
#    Copyright (C) 1997 Randy Paffenroth and John Maddocks
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU  General Public
#    License as published by the Free Software Foundation; either
#    version 2 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Library General Public License for more details.
#
#    You should have received a copy of the GNU Library General Public
#    License along with this library; if not, write to the Free
#    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
#    MA 02111-1307, USA

import os
import sys
from auto import AUTOExceptions
from auto import AUTOutil
from auto.AUTOutil import format19_10E3
try:
    from UserList import UserList
except ImportError: # Python 3
    from collections import UserList
from auto import Points

try:
    next
except NameError: #Python < 3
    def next(it):
        return it.next()

type_translation_dict = {
       0: {"long name" : "No Label","short name" : "No Label"},
       1: {"long name" : "Branch point (algebraic problem)","short name" : "BP"},
      -2: {"long name" : "Cusp (algebraic problem)","short name" : "CP"},
       2: {"long name" : "Fold (algebraic problem)","short name" : "LP"},
       3: {"long name" : "Hopf bifurcation (algebraic problem)","short name" : "HB"},
      -3: {"long name" : "Zero-Hopf bifurcation (algebraic problem)","short name" : "ZH"},
      -1: {"long name" : "Bogdanov-Takens bifurcation (algebraic problem)","short name" : "BT"},
      -32: {"long name" : "Generalized Hopf bifurcation (algebraic problem)","short name" : "GH"},
       4: {"long name" : "Regular point (every NPR steps)","short name" : "RG"},
      -4: {"long name" : "User requested point","short name" : "UZ"},
       5: {"long name" : "Fold (ODE)","short name" : "LP"},
       6: {"long name" : "Bifurcation point (ODE)","short name" : "BP"},
       7: {"long name" : "Period doubling bifurcation (ODE)","short name" : "PD"},
       8: {"long name" : "Bifurcation to invariant torus (ODE)","short name" : "TR"},
      -5: {"long name" : "1:1 Resonance bifurcation (ODE)","short name" : "R1"},
      -6: {"long name" : "1:2 Resonance bifurcation (ODE)","short name" : "R2"},
      -7: {"long name" : "1:3 Resonance bifurcation (ODE)","short name" : "R3"},
      -8: {"long name" : "1:4 Resonance bifurcation (ODE)","short name" : "R4"},
      23: {"long name" : "Fold-torus bifurcation (maps)","short name" :"LTR"},
      77: {"long name" : "Flip-torus bifurcation (maps)","short name" :"PTR"},
      28: {"long name" : "Fold-flip bifurcation (maps)","short name" :"LPD"},
      88: {"long name" : "Torus-torus bifurcation (maps)","short name" :"TTR"},
       9: {"long name" : "Normal begin or end","short name" : "EP"},
      -9: {"long name" : "Abnormal termination","short name" : "MX"}}

all_point_types = ["No Label",
                   "BP","LP","HB","BT","RG","UZ","PD","TR","EP","MX","ZH",
                   "CP","GH","R1","R2","R3","R4","LPD","LTR","PTR","TTR"]

def type_translation(type):
    """A little dictionary to transform types to human readable strings"""
    if type in [28,78]: # LPD
        type=28
    elif type in [23,83]: # LTR
        type=23
    elif type in [77,87]: # PTR
        type=77
    elif type not in [-32,88]:
        if type>=0:
            type=type%10
        else:
            type=-((-type)%10)
    return type_translation_dict.get(type,
                                     {"long name" : "Unknown type",
                                      "short name" : "Unknown type"})
    
def reverse_type_translation(type):
    """A little dictionary to transform human readable strings to types"""
    for k,v in type_translation_dict.items():
        if v["short name"] == type or v["long name"] == type:
            return k
    return type
    
# The parseB and AUTOBranch classes parse an AUTO fort.7 file
# THESE EXPECT THE FILE TO HAVE VERY SPECIFIC FORMAT!
# it provides 4 methods:
# read and write take as an arguement either and input or output
#    stream (basically any object with has the method "readline"
#    for reading and "write" for writing)
#    
# readFilename and writeFilename take as an arguement a filename
#    in which to read/write the parameters (basically it opens the
#    file and then calles "read" or "write"
#    
# Once the data is read in the class provides a list all the points
# in the fort.7 file.

class parseBMixin(object):
    def __str__(self):
        return self.summary()

    def __getitem__(self,index):
        return self.getIndex(index)

    def __call__(self,label=None):
        return self.getLabel(label)

    def dsp(self,label=None):
        """Removes solutions with the given labels or type names"""
        return self.deleteLabel(label,copy=1)

    def ksp(self,label=None):
        """Keeps solutions with the given labels or type names"""
        return self.deleteLabel(label,keep=1,copy=1)

    def dlb(self,label=None):
        """Removes solutions with the given labels or type names"""
        return self.deleteLabel(label,keepTY=1,copy=1)

    def klb(self,label=None):
        """Keeps solutions with the given labels or type names"""
        return self.deleteLabel(label,keepTY=1,keep=1,copy=1)

    def readFilename(self,filename):
        inputfile = AUTOutil.openFilename(filename,"r")
        self.read(inputfile)
        inputfile.close()

    def writeFilename(self,filename,append=False):
        if append:
            output = open(filename,"a")
        else:
            output = open(filename,"w")
        self.write(output)
        output.close()

    def writeRawFilename(self,filename):
        output = open(filename,"w")
        self.writeRaw(output)
        output.flush()
        output.close()
        
    def writeScreen(self):
        sys.stdout.write(self.summary())

# a point within an AUTOBranch
class BDPointData(UserList):
    def __init__(self, branch=None, idx=None):
        self.branch = branch
        self.idx = idx
    def __getattr__(self, attr):
        if attr == 'data':
            return [d[self.idx] for d in self.branch.coordarray]
        return super(BDPointData, self).__getattribute__(attr)
    def __setitem__(self, i, item):
        self.branch.coordarray[i][self.idx] = item
    def __str__(self):
        return str(self.data)
class BDPoint(Points.Point):
    def __init__(self, p, branch=None, idx=None, pt=None):
        if isinstance(p['coordarray'], list):
            # not fully parsed
            self.__fullyParsed = False
            self.p = p
            self.coordnames = p['coordnames']
            self.name_map = dict(zip(p['coordnames'],
                                     range(len(p['coordnames']))))
        else:
            self.__fullyParsed = True
            Points.Point.__init__(self, p)
        self.idx = idx
        self.pt = pt
        self.branch = branch
        self.section = 0
        self.index = idx
    
    def __getattr__(self, attr):
        if not self.__fullyParsed:
            p = self.p
            p['coordtype'] = type(p['coordarray'][0])
            del self.name_map, self.p
            Points.Point.__init__(self, p)
            self.__fullyParsed = True
        return super(BDPoint, self).__getattribute__(attr)

    def __contains__(self, key):
        if (key in ["TY","TY name","data","BR","PT","section","index"] or
            Points.Point.has_key(self,key)):
            return True
        for k,v in self.labels.items():
            if k in all_point_types:
                return key in v
        return False

    def has_key(self, key):
        return self.__contains__(key)

    def __setitem__(self, ixarg, val):
        """Change coordinate array values."""
        if type(ixarg) == type(""):
            if ixarg == "BR":
                self.branch["BR"] = val
                return
            elif ixarg == "PT":
                if not self.__fullyParsed:
                    self.pt = val
                # else ignore
                return
            elif ixarg == "section":
                self.section = val
                return
            elif ixarg == "index":
                self.index = val
                return
            for k,v in self.labels.items():
                if "LAB" in v:
                    v[ixarg] = val
                    return
        Points.Point.__setitem__(self, ixarg, val)

    def __getitem__(self, coords):
        if type(coords) == type(""):
            if not self.__fullyParsed:
                try:
                    return self.p['coordarray'][self.name_map[coords]]
                except KeyError:
                    pass
            if coords == "BR":
                return self.branch["BR"]
            elif coords == "PT":
                if self.__fullyParsed:
                    pt = self.idx+1
                    for p in self.branch.stability():
                        if abs(p) >= pt:
                            if p < 0:
                                pt = -pt
                            break
                    if pt < 0:
                        pt = -((-pt-1) % 9999) - 1
                    else:
                        pt = ((pt-1) % 9999) + 1
                else:
                    pt = self.pt
                return pt
            elif coords == "section":
                return self.section
            elif coords == "index":
                return self.index
            for k,v in self.labels.items():
                if k in all_point_types:
                    if coords in v:
                        return v[coords]
                    elif coords in ("TY name", "TY"):
                        return k
            if coords == "data":
                return BDPointData(self.branch, self.idx)
        return Points.Point.__getitem__(self, coords)

    def __str__(self):
        for k,v in self.labels.items():
            if k in all_point_types:
                ty_name = k
                label = v
                break
        return str({"BR": self["BR"],
                "PT": self["PT"],
                "TY number": label["TY number"],
                "TY name": ty_name,
                "LAB": label["LAB"],
                "data": list(self.coordarray),
                "section": self.section,
                "index": self.index})

    __repr__ = __str__

# a branch within the parseB class
class AUTOBranch(parseBMixin, Points.Pointset):
    def __init__(self,input=None,prevline=None,coordnames=[]):
        self.__fullyParsed = True
        if isinstance(input,AUTOBranch):
            for k,v in input.__dict__.items():
                self.__dict__[k] = v
        elif input is not None:
            self.read(input,prevline)
            self.__fullyParsed = False

    def __getattr__(self,attr):
        if self.__fullyParsed or attr == "__del__":
            raise AttributeError
        self.__parse()
        return super(AUTOBranch, self).__getattribute__(attr)

    def _gettypelabel(self,idx):
        for k,v in self.labels[idx].items():
            if "LAB" in v:
                return k,v
        return 'No Label', {}

    def __parse(self):
        if self.__fullyParsed:
            return
        global N
        if not Points.numpyimported:
            Points.importnumpy()
        self.__fullyParsed = True
        fromstring = Points.fromstring
        N = Points.N
        datalist = self.__datalist
        del self.__datalist
        line0 = datalist[0].split()
        self.BR = int(line0[0])
        ncolumns = len(line0)
        nrows = len(datalist)
        datalist = "".join(datalist)
        if fromstring: #numpy
            data = []
            if "D" not in datalist:
                data = fromstring(datalist, dtype=float, sep=' ')
            if len(data) != nrows * ncolumns:
                data = N.array(map(AUTOatof,datalist.split()), 'd')
            else:
                #make sure the last element is correct
                #(fromstring may not do this correctly for a
                #string like -2.05071-106)
                data[-1] = AUTOatof(datalist[datalist.rfind(' ')+1:].strip())
        else: #array
            datalist = datalist.split()
            try:
                data = N.array(map(float, datalist), 'd')
            except ValueError:
                data = N.array(map(AUTOatof, datalist), 'd')
        data.shape = (-1,ncolumns)
        coordarray = N.transpose(data[:,4:]).copy()
        points = data[:,1]
        if hasattr(N,"concatenate"):
            stability = self.__parsenumpy(points)
        else:
            stability = self.__parsearray(points)
        # add stability info labels
        branchtype = type_translation(self.TY)["short name"]
        for i in stability:
            if i < 0:
                stab = "S"
            else:
                stab = "U"
            self.labels.update(abs(i)-1, branchtype, {"stab": stab})
        # sometimes the columns names are the same: add spaces to those
        for i in range(len(self.coordnames)):
            name = self.coordnames[i]
            if self.coordnames.count(name) > 1:
                for j in range(i+1,len(self.coordnames)):
                    if self.coordnames[j] == name:
                        self.coordnames[j] = name + ' '
        Points.Pointset.__init__(self,{
            "coordarray": coordarray,
            "coordnames": self.coordnames,
            "labels": self.labels,
            })

    def __parsenumpy(self,points):
        global N
        # stability gives a list of point numbers where the stability
        # changes: the end point of each part is stored
        stab = N.concatenate((N.flatnonzero(N.less(points[:-1]*points[1:],0)),
                              [len(points)-1]))
        points = N.less(N.take(points,stab),0)
        stab = stab + 1
        stability = N.where(points,-stab,stab)
        return stability

    def __parsearray(self,points):
        global N
        # for those without numpy...
        stability = []
        prevpt = points[0]
        for p, pt in enumerate(points):
            if pt * prevpt < 0:
                if prevpt < 0:
                    p = -p
                stability.append(p)
            prevpt = pt
        p = len(points)
        if pt < 0:
            p = -p
        stability.append(p)
        return stability

    def __setitem__(self, item, value):
        if item in ("BR", "TY", "TY number") and item not in self.coordnames:
            if item == "BR":
                br = self["BR"]
                if value != br:
                    self.__parse()
                    self.BR = value
            else:
                if item == "TY":
                    value = reverse_type_translation(value)
                self.TY = value
                # sync solution TYs
                for k,x in map(self._gettypelabel, self.labels.getIndices()):
                    v = x["TY number"]
                    if v>=0:
                        v=v%10
                    else:
                        v=-((-v)%10)
                    x["TY number"] = value*10 + v
        else:
            Points.Pointset.__setitem__(self, item, value)

    def __len__(self):
        if not self.__fullyParsed:
            return len(self.__datalist)
        if len(self.coordarray) == 0:
            return 0
        return Points.Pointset.__len__(self)

    def deleteLabel(self,label=None,keepTY=0,keep=0,copy=0):
        """Removes solutions with the given labels or type names"""
        if label is None:
            label = [lab for lab in all_point_types if lab not in
                     ["No Label", "RG", "UZ"]]
        if isinstance(label, (str, int)):
            label = [label]
        if copy:
            new = self.__class__(self)
            new.labels = Points.PointInfo(self.labels.by_index.copy())
            if not self.__fullyParsed:
                new.__datalist = self.__datalist[:]
        else:
            new = self
        for idx in new.labels.getIndices():
            ty_name,v = new._gettypelabel(idx)
            if "LAB" not in v:
                continue
            if ((not keep and (v["LAB"] in label or ty_name in label)) or
               (keep and not v["LAB"] in label and not ty_name in label)):
                if copy:
                    new.labels[idx][ty_name] = v.copy()
                new.labels[idx][ty_name]["LAB"] = 0
                if not keepTY:
                    new.labels[idx][ty_name]["TY number"] = 0
                if not new.__fullyParsed:
                    new.__patchline(new.__datalist,idx,3,0)
                    if not keepTY:
                        new.__patchline(new.__datalist,idx,2,0)
                if v["TY number"] == 0:
                    new.labels.remove(idx)
        if copy:
            return new

    def relabel(self,old_label=1,new_label=None):
        """Relabels the first solution with the given label"""
        if new_label is None:
            label = old_label
            new = self.__class__(self)
            labels = {}
            if not self.__fullyParsed:
                new.__datalist = self.__datalist[:]
            for index in self.labels.getIndices():
                labels[index] = self.labels[index].copy()
                ty_name,v = self._gettypelabel(index)
                if v.get("LAB",0) != 0:
                    labels[index][ty_name] = v.copy()
                    labels[index][ty_name]["LAB"] = label
                    if not self.__fullyParsed:
                        self.__patchline(new.__datalist,index,3,label)
                    label = label + 1
            new.labels = Points.PointInfo(labels)
            return new
        labels = self.labels
        if isinstance(old_label, int):
            old_label = [old_label]
            new_label = [new_label]
        for j in range(len(old_label)):
            for index in self.labels.getIndices():
                v = self._gettypelabel(index)[1]
                if "LAB" not in v:
                    continue
                if v["LAB"] == old_label[j]:
                    v["LAB"] = new_label[j]
                    if not self.__fullyParsed:
                        self.__patchline(self.__datalist,index,3,new_label[j])

    def uniquelyLabel(self,label=1):
        """Make all labels in the file unique and sequential"""
        for index in self.labels.getIndices():
            v = self._gettypelabel(index)[1]
            if v.get("LAB",0) != 0:
                v["LAB"] = label
                if not self.__fullyParsed:
                    self.__patchline(self.__datalist,index,3,label)
                label = label + 1

    def getLabel(self,label):
        """Given a label, return the correct solution"""
        if label is None:
            return self
        if isinstance(label, int):
            for k in self.labels.getIndices():
                v = self._gettypelabel(k)[1]
                if v.get("LAB",0) == label:
                    return self.getIndex(k)
            raise KeyError("Label %s not found"%label)
        if isinstance(label, str) and len(label) > 2 and label[-1].isdigit():
            j = 2
            if not label[2].isdigit():
                j = 3
            number = int(label[j:])
            i = 0
            for k,v in self.labels.sortByIndex():
                if label[:j] in v:
                    i  = i + 1
                    if i == number:
                        return self.getIndex(k)
            raise KeyError("Label %s not found"%label)
        if not AUTOutil.isiterable(label):
            label = [label]
        labels = {}
        counts = [0]*len(label)
        for k,val in self.labels.sortByIndex():
            ty_name,v = self._gettypelabel(k)
            if "LAB" not in v:
                continue
            for i in range(len(label)):
                lab = label[i]
                j = 2
                if len(lab) > 2 and not lab[2].isdigit():
                    j = 3
                if (isinstance(lab, str) and len(lab) > j and
                    ty_name == lab[:j]):
                    counts[i] = counts[i] + 1
                    if counts[i] == int(lab[j:]):
                        labels[k] = val
            if v["LAB"] in label or ty_name in label:
                labels[k] = val
                continue
        new = self.__class__(self)
        new.labels = Points.PointInfo(labels)
        return new

    def getIndex(self,index):
        """Return a parseB style line item; if given a string, return the
        relevant column"""
        if isinstance(index,tuple) and len(index)==2 and not self.__fullyParsed:
            # retrieve data without parsing everything else
            i, j = index
            if isinstance(j, str):
                j = self.coordnames.index(j)
            return AUTOatof(self.__datalist[i].split()[4+j])
        if index in ("BR", "TY", "TY number") and index not in self.coordnames:
            if index == "BR":
                if self.__fullyParsed:
                    return self.BR
                else:
                    return int(self.__datalist[0].split(None,1)[0])
            elif index == "TY":
                return type_translation(self.TY)["short name"]
            else: #"TY number"
                return self.TY
        if isinstance(index, int) and index < 0:
            index = len(self) + index
        if self.__fullyParsed or not isinstance(index, int):
            if not Points.numpyimported:
                Points.importnumpy()
            ret = Points.Pointset.__getitem__(self,index)
            if isinstance(ret, Points.Pointset):
                r = self.__class__(self)
                r.coordarray = ret.coordarray
                r.labels = ret.labels
                # adjust stability info
                if isinstance(index, slice):
                    prev = 0
                    stability = []
                    for p in self.stability():
                        if p < 0:
                            stab = "S"
                        else:
                            stab = "U"
                        stability.append((abs(p)-prev)*stab)
                        prev = abs(p)
                    stability = "".join(stability)[index]
                    branchtype = type_translation(self.TY)["short name"]
                    prev = stability[0]
                    for i, stab in enumerate(stability):
                        if stab != prev:
                            r.labels.update(i-1, branchtype, {"stab": prev})
                        prev = stab
                    r.labels.update(len(r)-1, branchtype,{"stab":stability[-1]})
                return r
            if not isinstance(ret, Points.Point):
                return ret
            labels = ret.labels
            coordnames = ret.coordnames
            coordarray = ret.coordarray
            pt = None
        else:
            labels = self.labels[index]
            coordnames = self.coordnames
            coordarray = self.__datalist[index].split()
            pt = int(coordarray[1])
            coordarray = list(map(AUTOatof, coordarray[4:]))
        label = None
        for v in labels.values():
            if "LAB" in v:
                label = v
                break
        if label is None:
            label = {"TY number": 0, "LAB": 0}
            if "No Label" in labels:
                labels["No Label"].update(label)
            else:
                labels["No Label"] = label
        return BDPoint({'coordarray': coordarray,
                        'coordnames': coordnames,
                        'labels': labels},self,index)

    def getLabels(self):
        """Get all the labels from the solution"""
        labels = []
        for index in self.labels.getIndices():
            x = self._gettypelabel(index)[1]
            if x.get("LAB",0) != 0:
                labels.append(x["LAB"])
        return labels

    def stability(self):
        """Returns a list of point numbers where the stability
        changes: the end point of each part is stored."""
        if not self.__fullyParsed:
            self.__parse()
        stab = []
        prevpt = 0
        branchtype = type_translation(self.TY)["short name"]
        for idx,val in self.labels.sortByIndex():
            if branchtype in val and "stab" in val[branchtype]:
                if val[branchtype]["stab"] == "U":
                    pt = idx+1
                else:
                    pt = -idx-1
                if pt * prevpt <= 0:
                    stab.append(pt)
                else:
                    stab[-1] = pt
                prevpt = pt
        return stab

    def subtract(self,other,ref,pt=None):
        """Subtracts branch branches using interpolation with respect to other
        with monotonically increasing or decreasing reference coordinate ref,
        and starting point pt"""
        global N
        if not Points.numpyimported:
            Points.importnumpy()
        N = Points.N
        if pt is None:
            index = 0
        elif type(pt) == type(1):
            index = abs(pt) - 1
        else:
            index = pt["index"]
        coordarray = N.array(self.coordarray)
        if not isinstance(other,Points.Pointset):
            other = other[0]
        b0 = other[ref]
        if b0[index] > b0[index+1]:
            # decreasing array: take first part until it increases
            k = index+1
            while k < len(b0) and b0[k] <= b0[k-1]:
                k = k + 1
            b0 = b0[:k]
        else:
            k = index+1
            while k < len(b0) and b0[k] >= b0[k-1]:
                k = k + 1
            b0 = b0[:k]
        r = 0
        for i in range(len(other.coordnames)):
            if other.coordnames[i] == ref:
                r = i
        a0 = self.coordarray[r]
        k = index+1
        for j in range(len(a0)):
            if b0[index+1] > b0[index]:
                #find k so that b0[k-1] < a0[j] <= b0[k]
                if a0[j] > b0[k]:
                    while k < len(b0)-1 and a0[j] > b0[k]:
                        k = k + 1
                elif b0[k-1] >= a0[j]:
                    while k > index+1 and b0[k-1] >= a0[j]:
                        k = k - 1
            else:
                #find k so that b0[k-1] > a0[j] >= b0[k]
                if a0[j] < b0[k]:
                    while k < len(b0)-1 and a0[j] < b0[k]:
                        k = k + 1
                elif b0[k-1] <= a0[j]:
                    while k > index+1 and b0[k-1] <= a0[j]:
                        k = k - 1
            #do extrapolation if past the boundaries...
            for i in range(len(self.coordnames)):
                if i == r or i >= len(other.coordarray):
                    continue
                a = coordarray[i]
                b = other.coordarray[i]
                a[j]=a[j]-b[k-1]-(a0[j]-b0[k-1])*(b[k]-b[k-1])/(b0[k]-b0[k-1])
        new = self.__class__(self)
        Points.Pointset.__init__(new, coordarray = coordarray,
                                 coordnames = self.coordnames,
                                 labels = self.labels)
        return new

    def toArray(self):
        return list(map(list,zip(*self.coordarray)))

    def writeRaw(self,output):
        data = self.toArray()
        output.write("\n".join(["".join(["%24.15E"%v for v in d]) for d in data])+"\n")
                
    def write(self, output, columnlen=19):
        if columnlen == 19 and not self.__fullyParsed:
            output.writelines(self.headerlist)
            output.writelines(self.__datalist)
            return
        format = "%"+str(-columnlen)+"s"
        if self.headerlist != []:
            for l in self.headerlist:
                if " PT " not in l:
                    output.write(l)
        if self.headernames != []:
            output_line = ["   0    PT  TY  LAB "]
            for name in self.headernames:
                output_line.append(format%name)
            output.writelines(output_line)
            output.write('\n')
        br = self.BR
        data = self.coordarray
        istab = 0
        format = "%"+str(columnlen)+"."+str(columnlen-9)+"E"
        stability = self.stability()
        for i in range(len(data[0])):
            pt = i+1
            if stability[istab] < 0:
                pt = -pt
            tynumber = 0
            lab = 0
            if i in self.labels.by_index:
                for k,label in self.labels[i].items():
                    if "LAB" in label:
                        tynumber = label["TY number"]
                        lab = label["LAB"]
                        break
            if pt == stability[istab]:
                istab = istab + 1
            if pt < 0:
                pt = -((-pt-1) % 9999) - 1
            else:
                pt = ((pt-1) % 9999) + 1
            output_line = "%4d%6d%4d%5d"%(br,pt,tynumber,lab)
            if columnlen == 19:
                for j in range(len(data)):
                    output_line = output_line + format19_10E3(data[j,i])
            else:
                for j in range(len(data)):
                    output_line = output_line + format%data[j,i]
            output.write(output_line+"\n")

    def writeShort(self):
        self.write(sys.stdout, columnlen=14)

    def summary(self):
        slist = []
        if self.__fullyParsed:
            data = self.coordarray
        first = True
        for index,l in self.labels.sortByIndex():
            label = {}
            for k,v in l.items():
                if "LAB" in v:
                    label = v
                    break
            ty_number = label.get("TY number",0)
            if ty_number == 0 or label["LAB"] == 0:
                continue
            if first:
                slist.append("".join(
                        ["\n  BR    PT  TY  LAB "] +
                        ["%-14s"%name for name in self.headernames]))
                first = False
            ty_name = type_translation(ty_number)["short name"]
            if ty_name=='RG':
                ty_name = '  '
                
            if self.__fullyParsed:
                linelist = (["%4d%6d%4s%5d"%(abs(self.BR),(index % 9999) + 1,
                                             ty_name,label["LAB"])]+
                            ["%14.5E"%d[index] for d in data])
            else:
                linedata = self.__datalist[index].split()
                linelist = (
                    ["%4d%6d%4s%5d"%(abs(int(linedata[0])),abs(int(linedata[1])),
                                     ty_name,label["LAB"])] +
                    ["%14.5E"%AUTOatof(d) for d in linedata[4:]])
            slist.append("".join(linelist))
        return "\n".join(slist)

    def __patchline(self,datalist,lineno,column,new):
        #patch column of line with new integer value
        newsp = 0
        line = datalist[lineno]
        l = len(line)
        for i in range(column+1):
            oldsp = newsp
            start = l - len(line[oldsp:].lstrip())
            newsp = line.find(' ',start)
        datalist[lineno] = (line[:oldsp]+
                            '%'+str(newsp-oldsp)+'d'+line[newsp:])%new

    def __checknorotate(self,datalist):
        # Sometimes the point numbers rotate, like
        # 9996, 9997, 9998, 9999, 1, 2, ...
        # -9996, -9997, 1, 0, -1, -2, ... (an AUTO bug)
        # do not define a new branch if that happens
        prevpt = int(datalist[-2].split(None,2)[1])
        if prevpt not in [9999,-9999,9997,-9997,0]:
            return True
        # do some corrections
        if prevpt in [-9997,9997]:
            self.__patchline(datalist,-1,1,-9998)
        elif prevpt == 0:
            self.__patchline(datalist,-2,1,-9999)
        return False

    def read(self,inputfile,prevline=None):
        # We now go through the file and read the branches.
        # read the branch header
        # A section is defined as a part of a fort.7
        # file between "headers", i.e. those parts
        # of the fort.7 file which start with a 0
        # and contain information about the branch
        # FIXME:  I am not sure of this is the correct
        # fix to having multiple sections of a fort.7
        # file with the same branch number.  What it comes
        # dowm to is keeping the fort.7 and fort.8 files
        # in sync.  I.e. I could make sure that
        # this branch numbers are unique here, but then
        # the fort.7 file will not match the fort.8 file.
        # Another way for a section to start is with a point number
        # equal to 1.
        self._lastline = None
        if hasattr(str,"split"):
            split = str.split
        else:
            import string
            split = string.split
        if prevline:
            line = prevline
        else:
            try:
                line = next(inputfile)
            except StopIteration:
                self.labels = None
                return
        headerlist = []
        columns = split(line,None,2)
        if columns[0] == '0':
            headerlist.append(line)
            for line in inputfile:
                columns = split(line,None,2)
                if columns[0] != '0':
                    self._lastline = line
                    break
                headerlist.append(line)
        datalist = []
        labels = {}
        if columns[0] != '0':
            self._lastline = None
            datalist = [line]
            if columns[2][0] != '0': #type
                columns = split(line,None,4)
                pt = int(columns[1])
                if pt < 0:
                    stab = 'S'
                else:
                    stab = 'U'
                ty = int(columns[2])
                lab = int(columns[3])
                key = type_translation(ty)["short name"]
                labels[len(datalist)-1] = {key: {"LAB":lab,"TY number":ty}}
            for line in inputfile:
                datalist.append(line)
                columns = split(line,None,2)
                if (columns[0] == '0' or
                    ((columns[1] == '-1' or columns[1] == '1') and
                     self.__checknorotate(datalist))):
                    self._lastline = datalist.pop()
                    break
                if columns[2][0] != '0': #type
                    columns = split(datalist[-1],None,4)
                    pt = int(columns[1])
                    ty = int(columns[2])
                    lab = int(columns[3])
                    key = type_translation(ty)["short name"]
                    labels[len(datalist)-1] = {key: {"LAB":lab,"TY number":ty}}
        self.labels = Points.PointInfo(labels)
        self.__datalist = datalist
        self.c = self.parseHeader(headerlist)

    def parseHeader(self,headerlist):
        from auto import parseC
        self.headerlist = headerlist
        if hasattr(str,"split"):
            split = str.split
        else:
            import string
            split = string.split
        line = ""
        if headerlist != []:
            line = headerlist[-1]
        ncolumns = len(split(self.__datalist[0])) - 4
        self.headernames = []
        self.coordnames = []
        self.TY = 0
        if " PT " in line:
            linelen = len(self.__datalist[0])
            columnlen = (linelen - 19) // ncolumns
            n = linelen - columnlen * ncolumns
            for i in range(ncolumns):
                self.headernames.append(line[n:n+columnlen].rstrip())
                self.coordnames.append(line[n:n+columnlen].strip())
                n = n + columnlen
        if self.coordnames == []:
            self.coordnames = ["COLUMN(%d)"%(i+1) for i in range(ncolumns)]
        dict = parseC.parseC()
        i = 0
        if len(headerlist) == 0:
            return
        words = split(headerlist[0])
        if len(words) < 5:
            return
        for key in ["RL0","RL1","A0","A1"]:
            i = i + 1
            dict[key] = AUTOatof(words[i])
        key = ""
        userspec = False
        for line in headerlist[1:]:
            words = split(line)
            if len(words) < 2:
                break
            if words[-1] in ["constants:", "above:"]:
                userspec = True
                continue
            if words[1] == 'Branch':
                self.TY = int(words[3])
            if words[1] in ["User-specified", "Active"]:
                line = line.replace("s:",":")
                for ind in range(2,len(words)):
                    if words[ind] in ["parameter:","parameters:"]:
                        index = ind + 1
                        break
                if words[1][0] == "U":
                    key = "ICP"
                else:
                    key = "Active ICP"
                d = []
                for w in words[index:]:
                    try:
                        w = int(w)
                    except ValueError:
                        if ((w[0] == "'" and w[-1] == "'") or
                            (w[0] == '"' and w[-1] == '"')):
                            w = w[1:-1]
                    d.append(w)
                dict[key] = d
                continue
            dict.parseline(" ".join(words[1:]),userspec)
        return dict

class parseBR(parseBMixin, UserList):
    def __init__(self,filename=None):
        if isinstance(filename, str):
            UserList.__init__(self)
            self.readFilename(filename)
        else:
            UserList.__init__(self,filename)

    # Removes solutions with the given labels or type names
    def deleteLabel(self,label=None,keepTY=0,keep=0,copy=0):
        if copy:
            data = []
            for d in self.data:
                data.append(d.deleteLabel(label,keepTY,keep,copy))
            return self.__class__(data)
        for d in self.data:
            d.deleteLabel(label,keepTY,keep)
            
    # Relabels the first solution with the given label
    def relabel(self,old_label=None,new_label=None):
        if new_label is None:
            label = 1
            new = []
            for d in self.data:
                newd = d.relabel(label)
                for idx,val in newd.labels.sortByIndex():
                    for k,v in val.items():
                        if v.get("LAB",0) != 0:
                            label = v["LAB"]
                new.append(newd)
                label = label + 1
            return self.__class__(new)
        for d in self.data:
            d.relabel(old_label,new_label)

    # Make all labels in the file unique and sequential
    def uniquelyLabel(self):
        label = 1
        for d in self.data:
            d.uniquelyLabel(label)
            for idx,val in d.labels.sortByIndex():
                for k,v in val.items():
                    if v.get("LAB",0) != 0:
                        label = v["LAB"] + 1
            
    # Given a label, return the correct solution
    def getLabel(self,label):
        if isinstance(label, int):
            i = 0
            section = 0
            for d in self.data:
                l = len(d.coordarray[0])
                try:
                    item = d.getLabel(label)
                    item["index"] = item["index"] + i
                    item["section"] = section
                    return item
                except (KeyError, IndexError):
                    pass
                i = i + l
                section = section + 1
            raise KeyError("Label %s not found"%label)
        elif isinstance(label, str) and len(label) > 2 and label[-1].isdigit():
            j = 2
            if not label[2].isdigit():
                j = 3
            number = int(label[j:])
            for d in self.data:
                l = len(d.getLabel(label[:j]).getLabels())
                if number <= l:
                    return d.getLabel(label[:j]+str(number))
                number -= l
            raise KeyError("Label %s not found"%label)
        new = []
        label = label[:]
        for d in self.data:
            newbranch = d.getLabel(label)
            if newbranch:
                new.append(newbranch)
            for i, lab in enumerate(label):
                if isinstance(lab, str) and len(lab) > 2 and lab[-1].isdigit():
                    j = 2
                    if not lab[2].isdigit():
                        j = 3
                    number = int(lab[j:])
                    l = len(d.getLabel(lab[:j]).getLabels())
                    if number >= l:
                        number -= l
                        label[i] = lab[:j]+str(number)
        return self.__class__(new)

    def __getitem__(self,index):
        try:
            return UserList.__getitem__(self,index)
        except TypeError:
            return self.data[0][index]

    # Given an index, return the correct solution
    # Return a parseB style line item
    def getIndex(self,index):
        if type(index) != type(0):
            return self.data[0].getIndex(index)
        section = 0
        i = index
        for d in self.data:
            l = len(d)
            if i < l:
                item = d.getIndex(i)
                item["section"] = section
                item["index"] = index
                return item
            i = i - l
            section = section + 1
        raise IndexError

    # Get all the labels from the solution
    def getLabels(self):
        labels = []
        for d in self.data:
            labels.extend(d.getLabels())
        return labels

    # Merges branches
    def merge(self):
        fw = None
        data = []
        for bw in self.data:
            data.append(bw)
            if fw is None or fw.BR != bw.BR or (fw.c is not bw.c and
                                      fw.c["DS"] * bw.c["DS"] > 0):
                fw = bw
                continue
            f0 = fw[0]
            b0 = bw[0]
            if len(f0) != len(b0):
                fw = bw
                continue
            cont = False
            for i in range(len(f0)):
                if f0[i] != b0[i]:
                    fw = bw
                    cont = True
                    break
            if cont:
                continue
            #now we know that the branches have the same starting point:
            #merge them
            new = bw[::-1]
            new.extend(fw[1:])
            new.c = fw.c
            new.headerlist = fw.headerlist
            data[-1] = new

            # adjust label numbers
            lab = min(fw.getLabels()+bw.getLabels())
            for idx,val in new.labels.sortByIndex():
                for k,v in val.items():
                    if v.get("LAB",0) > 0:
                        val[k] = v.copy()
                        val[k]["LAB"] = lab
                        lab = lab+1

            if hasattr(fw,"diagnostics"):
                if hasattr(bw,"diagnostics"):
                    new.diagnostics = fw.diagnostics + bw.diagnostics
                else:
                    new.diagnostics = fw.diagnostics

            del data[-2]
            #reset for further search
            fw = None
        return self.__class__(data)

    def subtract(self,other,ref,pt=None):
        """Subtracts branch branches using interpolation with respect to other
        with monotonically increasing or decreasing reference coordinate ref,
        and starting point pt"""
        new = []
        for d in self.data:
            new.append(d.subtract(other,ref,pt))
        return self.__class__(new)

    def toArray(self):
        array = []
        for d in self.data:
            array.extend(d.toArray())
        return array

    def writeRaw(self,output):
        for d in self.data:
            d.writeRaw(output)
            output.write("\n")
                
    def write(self,output):
        for d in self.data:
            d.write(output)

    def writeShort(self):
        for d in self.data:
            d.writeShort()

    def summary(self):
        slist = []
        for branch in self.data:
            s = branch.__str__()
            if s != "":
                slist.append(s)
        return "\n".join(slist)+"\n"

    def __repr__(self):
        slist = []
        for branch in self.data:
            slist.append(branch.__repr__())
        return "\n".join(slist)+"\n"

    def read(self,inputfile):
        # We now go through the file and read the branches.
        prevline = None
        coordnames = []
        lastc = None
        lastTY = 0
        while True:
            branch = AUTOBranch(inputfile,prevline,coordnames)
            if branch.labels is None:
                break
            prevline = branch._lastline
            coordnames = branch.coordnames
            self.data.append(branch)
            if branch.c is None:
                #for header-less branches use constants of last branch header
                branch.c = lastc
                branch.TY = lastTY
            else:
                lastc = branch.c
                lastTY = branch.TY
            if prevline is None:
                break

class parseB(parseBMixin):
    #compatibility class for dg()
    def __init__(self,filename=None):
        self.branches = parseBR(filename)
        if len(self.branches) > 0:
            self.coordnames = self.branches[0].coordnames
        self.deleteLabel = self.branches.deleteLabel
        self.relabel = self.branches.relabel
        self.uniquelyLabel = self.branches.uniquelyLabel
        self.getIndex = self.branches.getIndex
        self.getLabels = self.branches.getLabels
        self.toArray = self.branches.toArray
        self.writeRaw = self.branches.writeRaw
        self.write = self.branches.write
        self.writeShort = self.branches.writeShort
        self.summary = self.branches.summary
    def __len__(self):
        l = 0
        for d in self.branches:
            l += len(d)
        return l
    def getLabel(self,label):
        if isinstance(label, int) or (
            isinstance(label, str) and len(label) > 2 and label[-1].isdigit()):
            return self.branches.getLabel(label)
        return self.__class__(self.branches.getLabel(label))
    def read(self,inputfile):
        self.branches.read(inputfile)
        if len(self.branches) > 0:
            self.coordnames = self.branches[0].coordnames
    def __repr__(self):
        return self.branches.__repr__()

def AUTOatof(input_string):
    #Sometimes AUTO messes up the output.  I.e. it gives an
    #invalid floating point number of the form x.xxxxxxxE
    #instead of x.xxxxxxxE+xx.  Here we assume the exponent
    #is 0 and make it into a real real number :-)
    try:
        return float(input_string)
    except ValueError:
        try:
            if not isinstance(input_string, str):
                input_string = input_string.decode('ascii')
            if input_string[-1] == "E":
                #  This is the case where you have 0.0000000E
                return float(input_string.strip()[0:-1])
            if len(input_string) >= 5:
                if input_string[-4] in ["-","+"]:
                    #  This is the case where you have x.xxxxxxxxx-yyy
                    #  or x.xxxxxxxxx+yyy (standard Fortran but not C)
                    return float(input_string[:-4]+'E'+input_string[-4:])
                if input_string[-4] == "D":
                    #  This is the case where you have x.xxxxxxxxxD+yy
                    #  or x.xxxxxxxxxD-yy (standard Fortran but not C)
                    return float(input_string[:-4]+'E'+input_string[-3:])
            input_string = input_string.replace("D","E")
            input_string = input_string.replace("d","e")
            try:
                return float(input_string)
            except ValueError:
                i = input_string.find("-", 1)
                if i == -1:
                    i = input_string.find("+", 1)
                if i != -1 and input_string[i-1] not in ["e", "E"]:
                    # form x.xxx+yy or x.xxx-yy (standard Fortran but not C)
                    return float(input_string[:i]+'E'+input_string[i:])
            print("Encountered value I don't understand")
            print(input_string)
            print("Setting to 0")
            return 0.0
        except ValueError:
            print("Encountered value which raises an exception while processing!!!")
            print(input_string)
            print("Setting to 0")
            return 0.0
            
            
def pointtest(a,b):
    if "TY name" not in a:
        raise AUTOExceptions.AUTORegressionError("No TY label")
    if "TY number" not in a:
        raise AUTOExceptions.AUTORegressionError("No TY label")
    if "BR" not in a:
        raise AUTOExceptions.AUTORegressionError("No BR label")
    if "data" not in a:
        raise AUTOExceptions.AUTORegressionError("No data label")
    if "PT" not in a:
        raise AUTOExceptions.AUTORegressionError("No PT label")
    if "LAB" not in a:
        raise AUTOExceptions.AUTORegressionError("No LAB label")
    if len(a["data"]) != len(b["data"]):
        raise AUTOExceptions.AUTORegressionError("Data sections have different lengths")

def test():
    print("Testing reading from a filename")
    foo = parseB()
    foo.readFilename("test_data/fort.7")    
    if len(foo) != 150:
        raise AUTOExceptions.AUTORegressionError("File length incorrect")
    pointtest(foo.getIndex(0),foo.getIndex(57))

    print("Testing reading from a stream")
    foo = parseB()
    fp = open("test_data/fort.7","r")
    foo.read(fp)
    if len(foo) != 150:
        raise AUTOExceptions.AUTORegressionError("File length incorrect")
    pointtest(foo.getIndex(0),foo.getIndex(57))


    print("Testing label manipulation")
    labels = foo.getLabels()
    foo.relabel(labels[0],57)
    labels = foo.getLabels()
    if labels[0] != 57:
        raise AUTOExceptions.AUTORegressionError("Error in either relabel or getLabel")
    foo.deleteLabel(labels[1])
    new_labels = foo.getLabels()
    if len(labels) != len(new_labels) + 1:
        raise AUTOExceptions.AUTORegressionError("Error in label deletion")
        
    bar = foo.relabel()
    old_labels = foo.getLabels()
    new_labels = bar.getLabels()
    if old_labels[0] != 57 or new_labels[0] != 1:
        raise AUTOExceptions.AUTORegressionError("Error in relabelling")
    print("parseB passed all tests")

if __name__ == '__main__' :
    test()











import os
from datetime import datetime as dt

from pandas import read_csv
from sympy.printing.fortran import FCodePrinter


class AutoCodePrinter(FCodePrinter):
    """Subclass of ``sympy.FCodePrinter`` with necessary parameters set
    for printing AUTO-07p FORTRAN code.

    See Also
    --------
    :doc:`Sympy Code Generation <sympy:modules/codegen>`
    """

    def __init__(self):
        settings = {"source_format": "free", "standard": 95}
        super().__init__(settings=settings)

    def _print_Symbol(self, expr):
        try:
            name = expr._name
        except AttributeError:
            name = expr.name
        return name

    def _print_Zero(self, expr):
        return "0.0d0"


class DataDir:
    def __init__(self, base_dir="./", name=""):
        self.date_time = dt.now().strftime(r"%Y%m%d_%H%M%S")
        self.directory = os.path.join(base_dir, f"{name}_{self.date_time}")
        self.codedir = self.directory + "code/"
        self.dir_created = False

    def create_dir(self):
        os.mkdir(self.directory)
        self.dir_created = True

    def dir(self):
        return self.directory

    def __str__(self):
        return self.directory

    def create_subdir(self, name):
        os.mkdir(os.path.join(self.directory, name))
        if not self.dir_created:
            self.dir_created = True
            os.mkdir(self.directory)
        return self.directory + "/" + name + "/"


class ParamDict(dict):
    def __str__(self):
        out = ""
        for name, val in self.items():
            out += f"{name:s}\t: {str(val):s}\n"
        return out


class AutoParameters(ParamDict):
    """Dictionary holding the AUTO-07p calculation parameters.
    Upon instantiation default values will be set.

    The following table gives a list of parameter names and their default
    values. The short explanations are taken from the
    `AUTO-07p manual <https://depts.washington.edu/bdecon/workshop2012/auto-tutorial/documentation/auto07p%20manual.pdf>`_.
    Please refer to the AUTO-07p manual for more detailed explanations.

    ========= ======= ===========
    Parameter Default Explanation
    ========= ======= ===========
    NTST      50      The number of mesh intervals used for discretization.
    IAD       3       This constant controls the mesh adaption.
                      ``IAD=3`` is the strongly recommended value.
    EPSL      1e-07   Relative convergence criterion for equation
                      parameters in the Newton/Chord method.
                      Recommended value range: ``10E-7`` to ``10E-6``.
    EPSU      1e-07   Relative convergence criterion for solution
                      components in the Newton/Chord method.
                      Recommended value range: ``10E-7`` to ``10E-6``.
    EPSS      1e-05   Relative arclength convergence criterion for the
                      detection of special solutions. Recommended value
                      range: ``10E-5`` to ``10E-4``. Generally, ``EPSS``
                      should be approximately 100 to 1000 times the value
                      of ``EPSL``, ``EPSU``.
    ITMX      8       The maximum number of iterations allowed in the
                      accurate location of special solutions, such as
                      bifurcations, folds, and user output points,
                      by Müller’s method with bracketing. The recommended
                      value is ``ITMX=8``
    ITNW      5       The maximum number of combined Newton-Chord
                      iterations. When this maximum is reached, the step
                      will be retried with half the stepsize. The
                      recommended value is ``ITNW=5``, but ``ITNW=7`` may
                      be used for “difficult” problems
    DS        0.1     Pseudo-arclength stepsize to be used for the first
                      attempted step along any family.
                      ``DSMIN`` ≤ | ``DS`` | ≤ ``DSMAX`` must hold.
    DSMIN     0.001   Minimum allowable absolute value of the
                      pseudo-arclength stepsize.
    DSMAX     0.2     Maximum allowable absolute value of the
                      pseudo-arclength stepsize.
    IADS      1       This constant controls the frequency of adaption of
                      the pseudo-arclength stepsize.

                            - ``IADS=0``: fixed pseudo-arclength stepsize ``DS``.
                            - ``IADS>0``: Adapt the pseudo-arclength stepsize after every ``IADS`` steps.
    ICP       [1]     For each equation type and for each continuation
                      calculation there is a typical (“ge
                      The array ICP designates these free parameters.
                      The parameter that appears first in the ICP list
                      is called the “principal continuation parameter”.
    STOP      []      This constant adds stopping conditions
    NMX       200     The maximum number of steps to be taken along any
                      family.
    RL0       0.0     The lower bound on the principal continuation
                      parameter. Typically this will be the load.
    RL1       0.0     The upper bound on the principal continuation
                      parameter.
    MXBF      10      Maximum number of bifurcations to be treated.
    NPR       200     This constant can be used to regularly write
                      ``fort.8`` plotting and restart data. IF ``NPR>0``
                      then such output is written every ``NPR`` steps.
                      If ``NPR=0`` or if ``NPR``≥``NMX`` then no such
                      output is written.
    IID       2       This constant controls the amount of diagnostic
                      output printed in ``fort.9``. See the AUTO-07p manual
                      for details.
    IPLT      0       This constant allows redefinition of the principal
                      solution measure, i.e. the error norm. See the
                      AUTO-07p manual for details.
    UZR       {}      This constant allows the setting of parameter values
                      which labeled plotting and restart information is to
                      be written in the ``fort.8`` output-file. See the
                      AUTO-07p manual for details.
    UZSTOP    {}      This constant specifies parameter values in the same
                      way as ``UZR`` above, but the computation will always
                      terminate if any solution point that is specified is
                      encountered.
    ========= ======= ===========
    """

    def __init__(self):
        default_parameters = {
            "NTST": 50,
            "IAD": 3,
            "EPSL": 1e-7,
            "EPSU": 1e-7,
            "EPSS": 1e-5,
            "ITMX": 8,
            "ITNW": 5,
            "DS": 0.1,
            "DSMIN": 1e-3,
            "DSMAX": 0.2,
            "IADS": 1,
            "ICP": [1],
            "STOP": "[]",
            "NMX": 200,
            "RL0": 0.0,
            "RL1": 0.0,
            "MXBF": 10,
            "NPR": 200,
            "IID": 2,
            "IPLT": 0,
            "UZR": "{}",
            "UZSTOP": "{}",
        }
        self.update(default_parameters)


class HiddenAutoParameters(ParamDict):
    def __init__(self):
        default_parameters = {
            "NBC": 0,
            "NINT": 0,
            "JAC": 0,
            "ILP": 0,
            "ISP": 1,
            "IRS": 0,
            "IPS": 0,
        }

        self.update(default_parameters)


class AutoOutputReader:
    def __init__(self, dirc):
        self.dirc = dirc
        self.outfile7 = os.path.join(self.dirc, "fort.7")

    def read_raw_data(self):
        # TODO Rewrite without pandas (Big chunky bloaty module for reading csv)
        line_numbers = self.find_table_lines()
        data = []
        for lines in line_numbers:
            start, stop = lines
            df = read_csv(
                self.outfile7,
                header=1,
                skiprows=start - 1,
                nrows=stop - start,
                delim_whitespace=True,
            )
            data.append(df)
        return data

    def find_table_lines(self):
        searching_for_start = 1
        line_numbers = []
        with open(self.outfile7) as data_file:
            for line_number, line in enumerate(data_file.readlines()):
                if searching_for_start:
                    if not line.lstrip().startswith("0"):
                        start_line = line_number - 1  # table header starts with 0
                        searching_for_start = 0
                else:
                    if line.lstrip().startswith("0"):
                        end_line = line_number - 1
                        line_numbers.append([start_line, end_line])
                        searching_for_start = 1
            if not searching_for_start:  # last table since there is no zero at the end
                line_numbers.append([start_line, line_number])
        return line_numbers



#! /usr/bin/env python

# This a class to parse fort.9 files, i.e. the diagnostics from AUTO
# NOTE:  This is nowhere near done, and it currently only finds the
# reduced jacobians and puts them into a list.

import re
import sys
try:
    from UserList import UserList
except ImportError: # Python 3
    from collections import UserList
import getopt
from auto import AUTOutil
from auto import parseB

class parseD(UserList):
    def __init__(self,filename=None):
        if isinstance(filename, str):
            UserList.__init__(self)
            self.readFilename(filename)
        else:
            UserList.__init__(self,filename)

    def __getitem__(self,index):
        return self.getIndex(index)

    def __call__(self,label):
        return self.getLabel(label)

    def __str__(self):
        s = []
        for d in self.data:
            s.append(d["Text"])
        return "".join(s)

    def getIndex(self,index):
        self.__readAll()
        return self.data[index]

    def getLabel(self,label):
        self.__readAll()
        for x in self.data:
            if x["Label"] == label:
                return x

    def __readAll(self):
        for item in self.data:
            solution = item["Text"]
            item.update({
                    "Branch number": 0,
                    "Point number": 0,
                    "Label": 0,
                    "Eigenvalues": [],
                    "Multipliers": []})
            lines = solution.splitlines()
            if len(lines) < 3:
                continue
            i = 0
            for line in lines:
                sp = line.split()
                if len(sp) > 0 and sp[0] == 'BR':
                    break
                i = i + 1
            if i + 1 >= len(lines):
                continue
            sp = lines[i+1].split()
            if len(sp) < 2:
                continue
            item["Branch number"] = int(sp[0])
            item["Point number"] = int(sp[1])
            labline = 0
            for line in lines:
                sp = line.split()
                if labline and len(sp) > 3:
                    if sp[2] != '0':
                        try:
                            item["Label"] = int(sp[2])
                        except:
                            item["Label"] = int(sp[3])
                    break
                if sp[0:4] == ['BR', 'PT', 'TY', 'LAB']:
                    labline = 1
            result = re.findall("Eigenvalue\s.*",solution)
            for eigenvalue_string in result:
                eigenvalue_string = eigenvalue_string.split()
                real_part = parseB.AUTOatof(eigenvalue_string[2])
                imag_part = parseB.AUTOatof(eigenvalue_string[3])
                item["Eigenvalues"].append([real_part,imag_part])
            result = re.findall("Multiplier\s.*",solution)
            for multiplier_string in result:
                multiplier_string = multiplier_string.split()
                # "inaccurate" or "accurate"
                if multiplier_string[1][-1] == "e":
                    continue
                real_part = parseB.AUTOatof(multiplier_string[2])
                imag_part = parseB.AUTOatof(multiplier_string[3])
                item["Multipliers"].append([real_part,imag_part])
        
    def read(self,input):
        data = input.read()
        divstr = "===============================================\n"
        data=data.split(divstr)
        if len(data) == 1:
            divstr = "--------------------------------------------------------------------------------------------\n"
            data=data[0].split(divstr)
        self.data=[]
        for solution in data:
            self.data.append({"Text": solution + divstr})
        self.data[-1]["Text"] = self.data[-1]["Text"][:-len(divstr)]

    def readFilename(self,filename):
        inputfile = AUTOutil.openFilename(filename, "r")
        self.read(inputfile)
        
    def write(self,output):
        output.write(self.__str__())

    def writeFilename(self,filename,append=False):
        if append:
            output = open(filename,"a")
        else:
            output = open(filename,"w")
        self.write(output)
        output.close()


def test():
    pass

if __name__ == "__main__":
    #Parse command line arguements
    opts_list,args=getopt.getopt(sys.argv[1:],"i:")
    opts={}
    for x in opts_list:
        opts[x[0]]=x[1]

    foo = parseD(args[0])
    foo.printMatrix(int(opts["-i"]),sys.stdout)

    





#! /usr/bin/env python
try:
    from ConfigParser import ConfigParser
except ImportError: # Python 3
    from configparser import ConfigParser
import os
import array
import gzip
import sys
from auto import AUTOExceptions
N = array

# This file contains code from the Python distribution.  As
# per the Python License we include the following:

##  -----------------------------------------------------------------------
##  Copyright 1991-1995 by Stichting Mathematisch Centrum, Amsterdam,
##  The Netherlands.

##                          All Rights Reserved

##  Permission to use, copy, modify, and distribute this software and its
##  documentation for any purpose and without fee is hereby granted,
##  provided that the above copyright notice appear in all copies and that
##  both that copyright notice and this permission notice appear in
##  supporting documentation, and that the names of Stichting Mathematisch
##  Centrum or CWI or Corporation for National Research Initiatives or
##  CNRI not be used in advertising or publicity pertaining to
##  distribution of the software without specific, written prior
##  permission.

##  While CWI is the initial source for this software, a modified version
##  is made available by the Corporation for National Research Initiatives
##  (CNRI) at the Internet address ftp://ftp.python.org.

##  STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
##  REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
##  MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
##  CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
##  DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
##  PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
##  TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
##  PERFORMANCE OF THIS SOFTWARE.
##  -----------------------------------------------------------------------

# Portions taken from SCons and AIMA:
# 
# http://code.google.com/p/aima-python/source/browse/trunk/utils.py
#
# __COPYRIGHT__
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

# Portions of the following are derived from the compat.py file in
# Twisted, under the following copyright:
#
# Copyright (c) 2001-2004 Twisted Matrix Laboratories

try:
    unicode
except NameError:  # 'unicode' is undefined, must be Python 3
    basestring = (str,bytes) 

def isiterable(value):
    # no strings!
    if isinstance(value, basestring): return False
    try:
        iter(value)
        return True
    except TypeError:
        return False

def findBaseClass(inputClass,baseClass):
    try:
        for base in inputClass.__bases__:
            if base == baseClass:
                return 1
            elif findBaseClass(base,baseClass) == 1:
                return 1
        return 0
        # Sometimes inputClass isn't really a class, if so we just return false
    except AttributeError:
        return 0

def getAUTORC(section=None):
    try:
        parser = ConfigParser(strict=False)
    except TypeError: # for Python 2.x without strict
        parser = ConfigParser()
    if section is not None:
        parser.add_section(section)
    path = os.path.expandvars("$AUTO_DIR/.autorc")
    if os.path.exists(path):
        parser.read(path)
    path = os.path.expandvars("$HOME/.autorc")
    if os.path.exists(path):
        parser.read(path)
    if os.path.exists("./autorc"):
        parser.read("./autorc")
    elif os.path.exists("./.autorc"):
        parser.read("./.autorc")
    return parser

def openFilename(filename,mode):
    try:
        inputfile = open(filename,mode)
    except IOError:
        s = sys.exc_info()[1]
        try:
            import gzip
            inputfile = gzip.open(filename+".gz",mode)
            inputfile.name = filename
        except IOError:
            raise IOError(s)
    return inputfile

def getstatusoutput(cmd, shell=False):
    try:
        import subprocess, os
        if not shell:
            for i in range(len(cmd)):
                cmd[i] = cmd[i].replace("'","")
        p = subprocess.Popen(cmd, shell=shell, stdout=subprocess.PIPE,
                             stderr=subprocess.STDOUT)
        output = p.communicate()
        s = output[0].decode('ascii')
        if len(s) > 0 and s[-1] == '\n':
            s = s[:-1]	
        return p.returncode, s
    except ImportError:
        import commands
        if isiterable(cmd):
            cmd = " ".join(cmd)
        return commands.getstatusoutput(cmd)

def format19_10E3(x):
    s = "%19.10E"%x
    if s[-4] == 'E':
        s = s[1:-2] + '0' + s[-2:]
    return s

try:
    import __builtin__
except ImportError:
    import builtins as __builtin__ # Python 3

try:
    raw_input
except NameError: # Python 3
    __builtin__.raw_input = input

try:
    all
except NameError:
    # Pre-2.5 Python has no all() function.
    def all(iterable):
        """
        Returns True if all elements of the iterable are true.
        """
        for element in iterable:
             if not element:
                return False
        return True
    __builtin__.all = all
    all = all
        
try:
    any
except NameError:
    # Pre-2.5 Python has no any() function.
    def any(iterable):
        """
        Returns True if any element of the iterable is true.
        """
        for element in iterable:
            if element:
                return True
        return False
    __builtin__.any = any
    any = any
        
try: reversed ## Introduced in 2.4
except NameError:
    def reversed(seq):
        """Iterate over x in reverse order.
        >>> list(reversed([1,2,3]))
        [3, 2, 1]
        """
        if hasattr(seq, 'keys'):
            raise ValueError("mappings do not support reverse iteration")
        i = len(seq)
        while i > 0:
            i -= 1
            yield seq[i]
    __builtin__.reversed = reversed


try: sorted ## Introduced in 2.4
except NameError:
    import copy
    def sorted(seq, cmp=None, key=None, reverse=False):
        """Copy seq and sort and return it.
        >>> sorted([3, 1, 2])
        [1, 2, 3]
        """    
        seq2 = copy.copy(seq)
        if key:
            if cmp == None:
                cmp = __builtins__.cmp
            seq2.sort(lambda x,y: cmp(key(x), key(y)))
        else:
            if cmp == None:
                seq2.sort()
            else:
                seq2.sort(cmp)
        if reverse:
            seq2.reverse()
        return seq2
    __builtin__.sorted = sorted

try:
    set, frozenset ## set builtin introduced in 2.4
except NameError:
    import sets ## sets module introduced in 2.3
    set, frozenset = sets.Set, sets.ImmutableSet
    __builtin__.set = set
    __builtin__.frozenset = frozenset
       
# very basic 1D/2D numpy emulation:
class array(object):
    def __init__(self, l, code=None, copy=True):
        self.base = 0
        if code is None:
            if isinstance(l, array):
                code = l.typecode
            else:
                code = 'd'
        if isinstance(l, array):
            if not copy:
                self.base = l.base
                self.typecode = l.typecode
                self.data = l.data
                self.__shape = l.shape
                self.strides = l.strides
                return
            l = l.tolist()
        if (isinstance(l, list) and len(l) > 0 and
            isinstance(l[0], (array, list))):
            a2 = []
            for a1 in l:
                if isinstance(a1, array):
                    a1 = a1.tolist()
                a2.extend(a1)
            self.data = N.array(code, a2)
            self.__shape = len(l), len(l[0])
            self.strides = len(l[0]), 1
        else:
            if not isiterable(l):
                l=[l]
            self.data = N.array(code, l) 
            self.__shape = len(self.data),
            self.strides = 1,
        self.typecode = code

    def __copy__(self):
        return array(self)

    copy = __copy__

    def __eq__(self, other):
        return array([a == b for a, b in zip(self, other)], 'B')

    def __ne__(self, other):
        return array([a != b for a, b in zip(self, other)], 'B')

    def __gt__(self, other):
        return array([a > b for a, b in zip(self, other)], 'B')

    def __lt__(self, other):
        return array([a < b for a, b in zip(self, other)], 'B')

    def __ge__(self, other):
        return array([a >= b for a, b in zip(self, other)], 'B')

    def __le__(self, other):
        return array([a <= b for a, b in zip(self, other)], 'B')

    def __add__(self, other):
        return array([a + other for a in self], self.typecode)

    def __mul__(self, other):
        return array([a * other for a in self], self.typecode)

    def __setattr__(self, attr, value):
        if attr == 'shape':
            length = 1
            for idx in self.__shape:
                length *= idx
            j = None
            for i, idx in enumerate(value):
                if idx == -1:
                    j = i
                elif idx != 0:
                    length //= idx
            if j is not None:
                value = value[:j] + (length,) + value[j+1:]
            self.__shape = value
            if len(value) == 2:
                self.strides = value[1], 1
            else:
                self.strides = 1,
        else:
            object.__setattr__(self, attr, value)

    def __getattr__(self, attr):
        if attr == 'shape':
            return self.__shape
        return object.__getattribute__(self, attr)

    def __subarray(self, i):
        if not isinstance(i, tuple):
            i = i,
        if len(i) < len(self.__shape):
            i = i + (slice(None),)
        base = self.base
        strides = []
        shape = []
        for j, idx in enumerate(i):
            if isinstance(idx, slice):
                idx, stop, step = idx.indices(self.__shape[j])
                strides.append(step*self.strides[j])
                shape.append(abs(stop-idx+step-1)//abs(step))
            else:
                if idx < 0:
                    idx += self.__shape[j]
                if idx < 0 or idx >= self.__shape[j]:
                    raise IndexError
            base += idx*self.strides[j]
        return base, strides, shape

    def __setitem__(self, i, v):
        base, strides, shape = self.__subarray(i)
        if len(strides) == 0:
            self.data[base] = v
        elif len(strides) == 1:
            stop = base+strides[0]*shape[0]
            if stop < 0:
                stop = None
            if isinstance(v,array):
                self.data[base:stop:strides[0]] = v._arrayslice()
            else:
                self.data[base:stop:strides[0]] = N.array(self.typecode, v)
        else:
            raise AUTOExceptions.AUTORuntimeError("2D updates only "
                                                  "supported with numpy.")

    def __getitem__(self, i):
        base, strides, shape = self.__subarray(i)
        if len(strides) == 0:
            return self.data[base]
        new = array(self, copy=False)
        new.base = base
        new.shape = tuple(shape)
        new.strides = tuple(strides)
        return new

    def __len__(self):
        return self.shape[0]

    def _arrayslice(self):
        stop = self.base+self.shape[0]*self.strides[0]
        if stop < 0:
            stop = None
        return self.data[self.base:stop:self.strides[0]]

    def __str__(self):
        return array2string(self)

    def tolist(self):
        if len(self.shape) == 1:
            return self._arrayslice().tolist()
        return [e.tolist() for e in self]

def rank(a):
    return len(a.shape)

def reshape(a,shape):
    new = array(a, copy=False)
    if isinstance(shape,int):
        shape = shape,
    new.shape = shape
    return new

def transpose(a):
    new = array(a, copy=False)
    new.shape = a.shape[::-1]
    new.strides = a.strides[::-1]
    return new

def take(a, idx, axis=0):
    try:
        if axis == 1:
            return array([[j[i] for i in idx] for j in a])
        return array([a[i] for i in idx])
    except TypeError:
        raise IndexError

def array2string(a,precision=0,format=None):
    indent = 1
    if a.typecode == 'd' and format is None:
        indent = 0
        absa = [abs(e) for e in ravel(a) if e != 0]
        if absa != []:
            minval = min(absa)
            maxval = max(absa)
        else:
            maxval = minval = 1
        if minval < 0.0001 or maxval > minval * 1000:
            maxlen = 16 + (0 < minval < 1e-99 or maxval >= 1e100)
            format = "%%%s.8e"%maxlen
        else:
            precision = 0
            for e in absa:
                s = "%.8f"%e
                precision = max(precision, 8 - (len(s) - len(s.rstrip("0"))))
            maxlen = len(str(int(maxval))) + precision + 2
            format = "%%#%s.%sf"%(maxlen, precision)
    if len(a.shape) == 2:
        return "["+"\n ".join([array2string(e,format=format)
                               for e in a])+"]"
    if a.typecode == 'B':
        return '[ '+"  ".join([str(e == 1) for e in a])+']'
    if a.typecode == 'd':
        ss = [format%e for e in a]
        if format[-1] == 'f':
            for i, s in enumerate(ss):
                t = s.rstrip("0")
                ss[i] = t + (len(s)-len(t))*" "
        if len(ss) > 0:
            itemsperline = 77//(1+len(ss[0]))
            for i in range(itemsperline-1, len(ss)-1, itemsperline):
                ss[i] = ss[i]+"\n"+" "*indent
        return '['+" ".join(ss)+']'
    return '[ '+"  ".join(map(str, a))+']'

def shape(a):
    try:
        return a.shape
    except AttributeError:
        return array(a).shape
        
def zeros(dim,code):
    if len(dim) == 1:
        return array(dim[0]*[0.0],code)
    return array([array(dim[1]*[0.0],code) for i in range(dim[0])],code)

def less(a, val):
    return array([v < val for v in a], a.typecode)

def ravel(a):
    if len(a.shape) == 1:
        return a
    new = array([])
    new.data = a.data
    new.shape = a.shape[0]*a.shape[1],
    new.strides = a.strides[1],
    new.base = a.base
    return new

ArrayType = array

def test():
    a=array([1,2,3,4])
    print("%s"%a[::-1])
    a=array([[1,2],[3,4]])
    print("%s %s"%(a[:,1],a[1,:]))
    print("%s"%a[:,::-1])
    print("%s"%a[::-1,:])
    print("%s"%a[::-1])
    print("%s"%a[-1,-1])
    print("%s"%a[::-1,::-1])
    b=array(a)
    b[0][0] = 4
    print("%s,%s"%(a,b))
    b[0,:]=[7,8]
    print("%s"%(b))

if __name__ == "__main__":
    test()



#!/usr/bin/env python
try:
    import Tkinter
    import tkSimpleDialog
    import tkFileDialog
except ImportError:
    import tkinter as Tkinter # Python 3
    from tkinter import simpledialog as tkSimpleDialog
    from tkinter import filedialog as tkFileDialog
from auto.graphics import Pmw
from auto import AUTOutil
from auto.graphics import optionHandler
import math
import sys
import string

GrapherError="GrapherError"
Axes3D=None

class BasicGrapher(optionHandler.OptionHandler,Tkinter.Canvas):
    """Documentation string for Basic Grapher

    A simple graphing widget
    By Randy P."""
    def __init__(self,parent=None,callback=None,**kw):
        self.data = []
        #Get the data from the arguments and then erase the
        #ones which are not used by canvas

        optionDefaults={}
        optionDefaults["minx"] = (0,callback)
        optionDefaults["maxx"] = (0,callback)
        optionDefaults["miny"] = (0,callback)
        optionDefaults["maxy"] = (0,callback)
        optionDefaults["minz"] = (0,callback)
        optionDefaults["maxz"] = (0,callback)
        optionDefaults["azimuth"] = (None,callback)
        optionDefaults["elevation"] = (None,callback)
        optionDefaults["left_margin"] = (80,callback)
        optionDefaults["right_margin"] = (40,callback)
        optionDefaults["top_margin"] = (40,callback)
        optionDefaults["bottom_margin"] = (40,callback)
        optionDefaults["decorations"] = (True,callback)
        optionDefaults["xlabel"] = (None,callback)
        optionDefaults["xlabel_fontsize"] = (None,callback)
        optionDefaults["ylabel"] = (None,callback)
        optionDefaults["ylabel_fontsize"] = (None,callback)
        optionDefaults["zlabel"] = (None,callback)
        optionDefaults["zlabel_fontsize"] = (None,callback)
        optionDefaults["xticks"] = (5,callback)
        optionDefaults["yticks"] = (5,callback)
        optionDefaults["zticks"] = (5,callback)
        optionDefaults["grid"] = (True,callback)
        optionDefaults["tick_label_template"] = ("%.2e",callback)
        optionDefaults["tick_length"] = (0.2,callback)
        optionDefaults["odd_tick_length"] = (0.4,callback)
        optionDefaults["even_tick_length"] = (0.2,callback)
        # background is handled by the Canvas widget
        optionDefaults["foreground"] = ("black",callback)
        optionDefaults["color_list"] = ("black red green blue",callback)
        optionDefaults["symbol_font"] = ("-misc-fixed-*-*-*-*-*-*-*-*-*-*-*-*",callback)
        optionDefaults["symbol_color"] = ("red",callback)
        optionDefaults["smart_label"] = (True,callback)
        optionDefaults["line_width"] = (2,callback)
        optionDefaults["realwidth"] = (1,callback)
        optionDefaults["realheight"] = (1,callback)
        optionDefaults["use_labels"] = (True,callback)
        optionDefaults["use_symbols"] = (True,callback)
        optionDefaults["top_title"] = ("",callback)
        optionDefaults["top_title_fontsize"] = (None,callback)
        optionDefaults["dashes"] = ((6.0,6.0),callback)
        optionDefaults["width"] = (600,callback)
        optionDefaults["height"] = (480,callback)

        optionAliases = {}
        optionAliases["fg"] = "foreground"
        # __parseOptions uses functions from the Canvas
        # widget, so we need to initialize it first
        if kw.get("hide") and 'graphics.grapher_mpl' in sys.modules:
            optionHandler.OptionHandler.__init__(self)
        else:
            try:
                Tkinter.Canvas.__init__(self,parent)
                optionHandler.OptionHandler.__init__(self,Tkinter.Canvas)
            except Tkinter.TclError:
                optionHandler.OptionHandler.__init__(self)

        for key in list(kw):
            if key not in optionDefaults:
                del kw[key]
        self.addOptions(**optionDefaults)
        self.addAliases(**optionAliases)
        BasicGrapher._configNoDraw(self,**kw)

    def __len__(self):
        return len(self.data)

    def config(self,cnf=None,**kw):
        rval = self._configNoDraw(cnf,**kw)
        if isinstance(cnf, str) or (cnf is None and not kw):
            return rval
        self.clear()
        self.draw()
    configure=config

    # This version can be used to increase efficiency
    # for example, if you want to config, but know you
    # will need to redraw later.
    def _configNoDraw(self,cnf=None,**kw):
        # note: reset xticks/yticks if min/max are set without ticks
        if (cnf is not None or kw) and not isinstance(cnf, str):
            dct = (cnf or {}).copy()
            dct.update(kw)
            for coord in ["x", "y", "z"]:
                minc = "min" + coord
                maxc = "max" + coord
                ticks = coord + "ticks"
                if (minc in dct or maxc in dct) and ticks not in dct:
                    dct[ticks] = None
            return optionHandler.OptionHandler.config(self,**dct)
        return optionHandler.OptionHandler.config(self,cnf,**kw)
    _configureNoDraw = _configNoDraw

    def _addData(self,data,newsect=None,color=None,stable=None):
        for array in data:
            if len(array[0]) != len(array[1]):
                raise GrapherError("Array lengths must match")
            new_array={}
            new_array["x"]=array[0]
            new_array["y"]=array[1]
            if len(array) > 2:
                new_array["z"]=array[2]
            new_array["stable"]=stable
            new_array["newsect"]=newsect
            new_array["color"]=color
            if len(array[0]) > 0:
                new_array["minx"]=min(array[0])
                new_array["maxx"]=max(array[0])
            if len(array[1]) > 0:
                new_array["miny"]=min(array[1])
                new_array["maxy"]=max(array[1])
            if "z" in new_array and len(array[2]) > 0:
                new_array["minz"]=min(array[2])
                new_array["maxz"]=max(array[2])
            self.data.append(new_array)
        
    def addData(self,data):
        self._addData(data)
        self.computeXRange()
        self.computeYRange()
        self.computeZRange()
        self.draw()

    def addArray(self,array):        
        self._addData((array,))
        self.computeXRange()
        self.computeYRange()
        self.computeZRange()
        self.draw()

    def addDataNoDraw(self,data):        
        self._addData(data)

    def addArrayNoDraw(self,array,newsect=None,color=None,stable=None):
        self._addData((array,),newsect,color,stable)

    def _delAllData(self):
        self.data=[]
        # check type for next data
        try:
            zcolumn = self.cget(self.cget("type")+"_z")
        except Tkinter.TclError: #in regression test
            return
        if zcolumn is not None:
            self._configNoDraw({self.cget("type")+"_z":None})
            print("\nWithout matplotlib 3D plots are not supported.")
            print("Plotting only the first two coordinates.")

    def delAllData(self):
        self._delAllData()
        self.clear()

    def _delData(self,index):
        del self.data[index]

    def delData(self,index):
        self._delData(index)
        self.clear()
        self.draw()

    def _round(self,val,increment):
        "This function returns the closest integer multiple to increment"
        quotient = val/increment
        remainder = quotient-math.floor(quotient)
        if remainder < 0.5:
            return math.floor(quotient)*increment
        else:
            return (math.floor(quotient)+1)*increment
        
    def _computeNiceRanges(self,minimum,maximum):
        # This bit of code computes "nice" range values.  Given a
        # minimum and manximum it computes a new minimum, maximum,
        # and number of divisions so that the number of digits in
        # the the numbers in the value at each tick mark
        # in minimized
        therange = maximum - minimum 
        inc = math.pow(10,math.ceil(math.log10(therange) - 1.0))
        if therange / inc <= 2:
            inc = inc / 4
        elif therange / inc <= 4:
            inc = inc / 2
        minimumr = self._round(minimum,inc)
        if minimumr > minimum:
            minimum = minimumr - inc
        else:
            minimum = minimumr
        maximumr = self._round(maximum,inc)
        if maximumr < maximum:
            maximum = maximumr + inc
        else:
            maximum = maximumr ;
        num = int(round(( maximum - minimum ) / inc))
        return {"min": minimum, "max": maximum, "divisions": num + 1}

    def computeRange(self,coordinate,guess_minimum=None,guess_maximum=None):
        minc = "min"+coordinate
        maxc = "max"+coordinate
        if len(self.data) > 0 and minc not in self.data[0]:
            return
        if guess_minimum is None and len(self.data) > 0:
            guess_minimum = min([entry[minc] for entry in self.data])

        if guess_maximum is None and len(self.data) > 0:
            guess_maximum = max([entry[maxc] for entry in self.data])

        if guess_minimum != guess_maximum:
            d = self._computeNiceRanges(guess_minimum,guess_maximum)
            self._configNoDraw(**{minc:d["min"],maxc:d["max"],
                                  coordinate+'ticks':d["divisions"]})
        elif guess_maximum != None:
            self._configNoDraw(**{minc:guess_minimum-1,maxc:guess_maximum+1})
            
    def computeXRange(self,guess_minimum=None,guess_maximum=None):
        self.computeRange("x",guess_minimum,guess_maximum)

    def computeYRange(self,guess_minimum=None,guess_maximum=None):
        self.computeRange("y",guess_minimum,guess_maximum)

    def computeZRange(self,guess_minimum=None,guess_maximum=None):
        self.computeRange("z",guess_minimum,guess_maximum)

    def getXRange(self):
        return [self.cget("minx"),self.cget("maxx")]

    def getYRange(self):
        return [self.cget("miny"),self.cget("maxy")]

    def getZRange(self):
        return [self.cget("minz"),self.cget("maxz")]

    def clear(self):
        if hasattr(self, "tk"):
            for x in self.find_all():
                self.delete(x)

    def plot(self):
        pass

    def draw(self):
        color_list = self.cget("color_list").split()
        minx=self.cget("minx")
        maxx=self.cget("maxx")
        miny=self.cget("miny")
        maxy=self.cget("maxy")
        top_margin = self.cget("top_margin")
        bottom_margin = self.cget("bottom_margin")
        left_margin = self.cget("left_margin")
        right_margin = self.cget("right_margin")
        width = int(self.cget("realwidth"))
        height = int(self.cget("realheight"))

        if self.cget("decorations"):
            # border
            self.create_polygon(left_margin,top_margin,
                                    int(width)-right_margin,top_margin,
                                    int(width)-right_margin,int(height)-bottom_margin,
                                    left_margin,int(height)-bottom_margin,fill="",outline=self.cget("foreground"))
        # data
        line_width=self.cget("line_width")
        adjwidth = width - (left_margin + right_margin)
        adjheight = height - (top_margin + bottom_margin)
        xscale = (float(maxx) - minx) / adjwidth
        yscale = (float(maxy) - miny) / adjheight
        i=-1
        for d in self.data:
            if d["newsect"] is None or d["newsect"]:
                i = i+1
            if d["color"] is None:
                color = i
            else:
                color = d["color"]
            fill=color_list[color%len(color_list)]
            curve="curve:%d"%(i,)
            n=len(d["x"])
            [x,y]=self.__valueToCanvasFast([d["x"][0],d["y"][0]],minx,maxx,miny,maxy,
                                            width,height,left_margin,right_margin,top_margin,bottom_margin)
            # If we only have one point we draw a small circle
            if n == 1:
                self.create_oval(x-3,y-3,x+3,y+3,
                                 tags=("data_point:%d"%(0,),"curve:%d"%(i,),"data"),
                                 fill=fill)
            else:
                line = [x, y]
                xs = d["x"]
                ys = d["y"]
                stable = d["stable"]
                for j in range(1, n):
                    line.append((xs[j]-minx) / xscale + left_margin)
                    line.append((adjheight - (ys[j]-miny) / yscale + top_margin))
                if stable is None or stable:
                    self.create_line(line,width=line_width,tags=(curve,"data"),fill=fill)
                else:
                    self.create_line(line,width=line_width,tags=(curve,"data"),fill=fill,dash=(10,10))

            
        if self.cget("decorations"):
            # clip stuff outside box
            self.create_polygon(0,0,
                                int(width),0,
                                int(width),top_margin-1,
                                0,top_margin-1,
                                fill=self["background"])
            self.create_polygon(0,0,
                                0,int(height),
                                left_margin-1,int(height),
                                left_margin-1,0,
                                fill=self["background"])
            self.create_polygon(int(width),int(height),
                                int(width),0,
                                int(width)-right_margin+1,0,
                                int(width)-right_margin+1,int(height),
                                fill=self["background"])
            self.create_polygon(int(width),int(height),
                                0,int(height),
                                0,int(height)-bottom_margin+1,
                                int(width),int(height)-bottom_margin+1,
                                fill=self["background"])

            # tick marks
            xw=float(width) - (float(left_margin) + float(right_margin))
            yw=float(height) - (float(top_margin) + float(bottom_margin))

            tick_label_template=self.cget("tick_label_template")
            tick_length=self.cget("tick_length")
            odd_tick_length=self.cget("odd_tick_length")
            even_tick_length=self.cget("even_tick_length")

            xticks = self.cget("xticks")
            if xticks is None:
                xticks = self.config("xticks")[3]
            else:
                xticks=int(xticks)
            tick_start_y=yw+bottom_margin
            for i in range(xticks):
                # The odd tick marks should be longer
                if i%2==0:
                    tick_end_y=yw+bottom_margin*(1+even_tick_length)
                else:
                    tick_end_y=yw+bottom_margin*(1+odd_tick_length)
                    
                tick_x=left_margin + float(i)*xw/float(xticks-1)
                self.create_line(tick_x,tick_start_y,tick_x,tick_end_y,fill=self.cget("foreground"))
                val = self.canvasToValue((tick_x,tick_start_y))
                self.create_text(tick_x,tick_end_y,text=tick_label_template%(val[0],),anchor="n",fill=self.cget("foreground"))
                if i != 0 and i != xticks - 1 and self.cget("grid") in ["yes",True]:
                    self.create_line(tick_x,tick_start_y,tick_x,tick_start_y-yw,
                                     fill=self.cget("foreground"),stipple="gray50")
            yticks = self.cget("yticks")
            if yticks is None:
                yticks = self.config("yticks")[3]
            else:
                yticks=int(yticks)
            tick_start_x=left_margin
            tick_end_x=left_margin*(1-tick_length)
            for i in range(yticks):
                tick_y=bottom_margin + float(i)*yw/float(yticks-1)
                self.create_line(tick_start_x,tick_y,tick_end_x,tick_y,fill=self.cget("foreground"))
                val = self.canvasToValue((tick_start_x,tick_y))
                self.create_text(tick_end_x,tick_y,text=tick_label_template%(val[1],),anchor="e",fill=self.cget("foreground"))
                if i != 0 and i != yticks - 1 and self.cget("grid") in ["yes",True]:
                    self.create_line(tick_start_x,tick_y,tick_start_x + xw,tick_y,
                                     fill=self.cget("foreground"),stipple="gray50")

            # Axis labels
            self.create_text(left_margin*0.3,bottom_margin*0.3,
                             text=self.cget("ylabel"),anchor="nw",fill=self.cget("foreground"))
            self.create_text(int(width)-left_margin*0.3,int(height)-bottom_margin*0.1,
                             text=self.cget("xlabel"),anchor="se",fill=self.cget("foreground"))
            # Title
            self.create_text((left_margin-right_margin+int(width))//2,
                             top_margin*0.1,text=self.cget("top_title"),anchor="n",
                             fill=self.cget("foreground"))

        
    def valueToCanvas(self,val):
        if len(val) != 2:
            raise GrapherError("Illegal value choosen for coordinate transformation.  Must be a tuple with 2 elements.")
        # make a few constants shorter
        minx=self.cget("minx")
        maxx=self.cget("maxx")
        miny=self.cget("miny")
        maxy=self.cget("maxy")
        width = int(self.cget("realwidth"))
        height = int(self.cget("realheight"))
        left_margin = self.cget("left_margin")
        right_margin = self.cget("right_margin")
        top_margin = self.cget("top_margin")
        bottom_margin = self.cget("bottom_margin")
        return self.__valueToCanvasFast(val,minx,maxx,miny,maxy,
                                        width,height,left_margin,right_margin,top_margin,bottom_margin)

    def __valueToCanvasFast(self,val,minx,maxx,miny,maxy,
                            width,height,left_margin,right_margin,top_margin,bottom_margin):
        x = val[0]
        y = val[1]
        width = width - (left_margin + right_margin)
        height = height - (top_margin + bottom_margin)
        return [((x-minx)/(maxx-minx))*width + left_margin,
                height - ((y-miny)/(maxy-miny))*height + top_margin]
        
    def transform(self,val):
        [x,y] = self.valueToCanvas(val)
        return [x,self.cget("realheight") - y]

    def transform_seq(self,seqs):
        minx=self.cget("minx")
        maxx=self.cget("maxx")
        miny=self.cget("miny")
        maxy=self.cget("maxy")
        width = int(self.cget("realwidth"))
        height = int(self.cget("realheight"))
        left_margin = self.cget("left_margin")
        right_margin = self.cget("right_margin")
        top_margin = self.cget("top_margin")
        bottom_margin = self.cget("bottom_margin")
        valuetocanvasfast = self.__valueToCanvasFast
        sp2 = 5 #fontsize
        sp4 = 5
        l = []
        for i in range(len(seqs[0])):
            val = [seqs[0][i],seqs[1][i]]
            [x,y] = valuetocanvasfast(val,minx,maxx,miny,maxy,
                width,height,left_margin,right_margin,top_margin,bottom_margin)
            l.append(
                [(x - left_margin) / sp2,((height - y) - bottom_margin) / sp4])
        return l

    def canvasToValue(self,val):
        if len(val) != 2:
            raise GrapherError("Illegal value choosen for coordinate transformation.  Must be a tuple with 2 elements.")
        x = val[0]
        if x < self.cget("left_margin"):
            x = self.cget("left_margin")
        if x > int(self.cget("realwidth")) - self.cget("right_margin"):
            x = int(self.cget("realwidth")) - self.cget("right_margin")
        y = val[1]
        if y < self.cget("top_margin"):
            y = self.cget("top_margin")
        if y > int(self.cget("realheight")) - self.cget("bottom_margin"):
            y = int(self.cget("realheight")) - self.cget("bottom_margin")

        x = x - self.cget("left_margin")
        y = y - self.cget("bottom_margin")
        minx=self.cget("minx")
        maxx=self.cget("maxx")
        miny=self.cget("miny")
        maxy=self.cget("maxy")
        return [minx + (maxx-minx)*float(x)/(float(self.cget("realwidth"))-(self.cget("left_margin")+self.cget("right_margin"))),
                maxy - (maxy-miny)*float(y)/(float(self.cget("realheight"))-(self.cget("bottom_margin")+self.cget("top_margin")))]

class LabeledGrapher(BasicGrapher):
    def __init__(self,parent=None,**kw):
        self.labels=[]
        BasicGrapher.__init__(self,parent,**kw)

    def addLabel(self,i,j,input_text,symbol=None):
        new_label={}
        new_label["j"]=j
        xy = [self.data[i]["x"][j], self.data[i]["y"][j]]
        if "z" in self.data[i]:
            xy.append(self.data[i]["z"][j])
        new_label["xy"]=xy
        new_label["text"]=input_text
        new_label["symbol"]=symbol
        self.labels[i].append(new_label)

    def _delData(self,i):
        del self.labels[i]
        BasicGrapher._delData(self,i)

    def _delAllData(self):
        self.labels=[]
        BasicGrapher._delAllData(self)

    def _addData(self,data,newsect=None,color=None,stable=None):
        self.labels.append([])
        BasicGrapher._addData(self,data,newsect,color,stable)

    def plotlabels(self):
        if self.cget("realwidth") == 1 or self.cget("realheight") == 1:
            return

        if not self.cget("use_labels") or len(self.labels) == 0:
            return

        trans = self.transform_seq
        if self.cget("smart_label"):
            mp = self.inarrs()
            for d in self.data:
                self.map_curve(mp,trans([d["x"],d["y"]]))
        for i, labels in enumerate(self.labels):
            for label in labels:
                if len(label["text"]) == 0:
                    continue
                [x,y] = label["xy"][:2]
                if (x < self["minx"] or x > self["maxx"] or
                    y < self["miny"] or y > self["maxy"]):
                    continue
                data = self.transform([x,y])
                if not(data is None):
                    [x,y] = data
                    if self.cget("smart_label"):
                        [xoffd1,yoffd1,xoffd2,yoffd2,
                         xofft,yofft,pos] = self.findsp(x,y,mp)
                    else:
                        [xoffd1,yoffd1,xoffd2,yoffd2,
                         xofft,yofft,pos] = self.dumblabel(i,label["j"],x,y)
                    anchor = self.getpos(pos)
                    y = self.cget("realheight") - y
                    self.create_line(x+xoffd1,y-yoffd1,x+xoffd2,y-yoffd2,
                                     fill=self.cget("foreground"))
                    self.create_text(x+xofft,y-yofft,text=label["text"],
                                     anchor=anchor,fill=self.cget("foreground"))

    # not-so-smart way of plotting labels
    def dumblabel(self,i,j,x,y):
        #Find a neighbor so I can compute the "slope"
        if j < len(self.data[i]["x"]) - 1:
            first = j
            second = j+1
        else:
            first = j-1
            second = j
        realwidth=self.cget("realwidth")
        realheight=self.cget("realheight")
        left_margin=self.cget("left_margin")
        top_margin=self.cget("top_margin")
        #pick a good direction for the label
        ys = self.data[i]["y"]
        if ys[second] > ys[first]:
            if (x < int(realwidth)-(20+left_margin) and
                y < int(realheight)-(20+top_margin)):
                xoffset = 10
                yoffset = 10
                pos=0
            else:
                xoffset = -10
                yoffset = -10
                pos=4
        else:
            if x > 20+left_margin and y < int(realheight)-(20+top_margin):
                xoffset = -10
                yoffset = 10
                pos=2
            else:
                xoffset = 10
                yoffset = -10
                pos=6

        #self.addtag_overlapping("overlaps",x+xoffset-3,
        #        y+yoffset-3,x+xoffset+3,y+yoffset+3)
        #if len(self.gettags("overlaps")) != 0:
        #    print self.gettags("overlaps")
        #self.dtag("overlaps")
        #print "---------------------------------------------"    
        return [xoffset//10,yoffset//10,xoffset,yoffset,xoffset,yoffset,pos]

    # smarter way to plot labels: ported from old PLAUT
    #-------------------------------------------------------------------
    #   Tries to find an empty space to put the
    #   point or branch label from pnts in by searching for three
    #   unchanged entries in mp.  The search is outward from the
    #   point to be labelled.  If space can't be found the label
    #   will be written out anyway.
    #-------------------------------------------------------------------
    def findsp(self,x1,y1,mp):
        sp1 = self.cget("left_margin")
        minsx = sp1
        maxsx = self.cget("realwidth") - self.cget("right_margin")
        sp2 = 5 #fontsize
        sp3 = self.cget("bottom_margin")
        minsy = sp3
        maxsy = self.cget("realheight") - self.cget("top_margin")
        sp4 = 5
        #C---     *x1, y1 ARE SCREEN COORDINATES
        xi = (x1 - sp1) / sp2
        yi = (y1 - sp3) / sp4
        pi2 = math.pi * 2.0
        start = pi2 / 12.0
        npoint = 16

        hx = len(mp) - 5
        hy = len(mp[0]) - 5
        found = False
        for radius in range(3,71):
            npoint = npoint + 8

            for st in [start + (float(i)/npoint)*pi2 for i in range(npoint)]:
                xd = int(xi + radius * math.cos(st))
                yd = int(yi + radius * math.sin(st))
                #-------------------------------------------------------
                #   Organize the search along two sides of
                #   a square of size 2(xd) with pnt xi, yi in its center.
                #--------------------------------------------------------
                if xd <= hx and xd >= 3 and yd <= hy and yd >= 3:
                    found = True
                    for col in mp[xd-2:xd+3]:
                        if max(col[yd-2:yd+3]) == 1:
                            found = False
                            break
                    if found:
                        r1 = (radius + 3) * sp2
                        ix = x1 + r1 * math.cos(st)
                        iy = y1 + r1 * math.sin(st)
                        if ix>=minsx and ix<=maxsx and iy>=minsy and iy<=maxsy:
                            break
                        found = False
            if found:
                break

        if not found:
            return [1,1,10,10,11,11,4] #if no empty space

        for col in mp[xd-2:xd+3]:
            for i2 in range(yd-1,yd+2):
                col[i2] = 1

        pos = int(st/pi2*8.0 - 0.5)
        radius = radius * sp2
        cosst = math.cos(st)
        sinst = math.sin(st)
        d1dist = self.cget("line_width") + 1
        xoffd1 = d1dist * cosst
        yoffd1 = d1dist * sinst
        xoffd2 = radius * cosst
        yoffd2 = radius * sinst
        xofft = (radius + 2) * cosst
        yofft = (radius + 2) * sinst
        return [xoffd1,yoffd1,xoffd2,yoffd2,xofft,yofft,pos]

    def getpos(self,pos):
        anchor = [ "sw", "s", "se", "e", "ne", "n", "nw", "w", "sw" ]
        return anchor[pos]

    #-----------------------------------------------------------------------
    #    Initializes the map array that is
    #    used to label the plotted curves.
    #-----------------------------------------------------------------------
    def inarrs(self):
        sp1 = self.cget("left_margin")
        sp2 = 5 #fontsize
        sp3 = self.cget("bottom_margin")
        sp4 = 5
        nx = int(self.cget("realwidth")-sp1-self.cget("right_margin"))//sp2
        ny = int(self.cget("realheight")-sp3-self.cget("top_margin"))//sp4
        r = ny*[0]
        return [r[:] for i in range(nx)]

    #-----------------------------------------------------------------------
    #        Maps the curves in mp array
    #-----------------------------------------------------------------------
    def map_curve(self,mp,xys):
        ixmax = len(mp)
        iymax = len(mp[0])
        [x2,y2] = xys[0]
        ix2,iy2 = int(x2),int(y2)
        for xy in xys[1:]:
            x1,y1 = x2,y2
            ix1,iy1 = ix2,iy2
            [x2,y2] = xy
            ix2,iy2 = int(x2),int(y2)
            if ix1 == ix2 and iy1 == iy2:
                #optimize common case of short line segments
                if ix1 >= 0 and ix1 < ixmax and iy1 >= 0 and iy1 < iymax:
                    mp[ix1][iy1] = 1
                continue
            # if we are sure that the line is outside the graph limits we
            # should skip and save a lot of time
            if ((ix1 < 0 and ix2 < 0) or (iy1 < 0 and iy2 < 0) or
                (ix1 >= ixmax and ix2 >= ixmax) or 
                (iy1 >= iymax and iy2 >= iymax)):
                continue
            dx = x2 - x1
            dy = y2 - y1
            index = int(max(abs(dx),abs(dy))) + 1
            ilow  = 0
            ihigh = index
            if dx != 0:
                xlim1 = -x1*index/dx
                xlim2 = (ixmax-x1)*index/dx+1
                ilow  = max(ilow,min(xlim1,xlim2))
                ihigh = min(ihigh,max(xlim1,xlim2))
            if dy != 0:
                ylim1 = -y1*index/dy
                ylim2 = (iymax-y1)*index/dy+1
                ilow  = max(ilow,min(ylim1,ylim2))
                ihigh = min(ihigh,max(ylim1,ylim2))
            for i in range(int(ilow),int(ihigh)):
                f = float(i)/index
                ix = int(x1 + f * dx)
                iy = int(y1 + f * dy)
                try:
                    mp[ix][iy] = 1
                except IndexError:
                    pass

    def draw(self):
        self.plotlabels()
        BasicGrapher.draw(self)
        self.plotsymbols()

    def plotsymbols(self):
        if not self.cget("use_symbols"):
            return

        for i, labels in enumerate(self.labels):
            for label in labels:
                l = label["symbol"]
                if l is None:
                    continue
                data = self.valueToCanvas(label["xy"][:2])
                if data is None:
                    continue
                [x,y] = data
                if (x < self["minx"] or x > self["maxx"] or
                    y < self["miny"] or y > self["maxy"]):
                    continue
                c = self.cget("symbol_color")
                if len(l) <= 3:
                    self.create_text(x,y,font=self.cget("symbol_font"),
                                     fill=c,text=l)
                    continue
                ms = 3
                for i, ch in enumerate(l):
                    if ch in string.digits:
                        ms = int(l[i:])
                        l = l[:i]
                        break
                if l == "fillcircle":
                    self.create_oval(x-ms,y-ms,x+ms,y+ms,fill=c,outline=c)
                elif l == "circle":
                    self.create_oval(x-ms,y-ms,x+ms,y+ms,outline=c)
                elif l == "square":
                    self.create_rectangle(x-ms,y-ms,x+ms,y+ms,outline=c)
                elif l == "crosssquare":
                    self.create_rectangle(x-ms,y-ms,x+ms,y+ms,outline=c)
                    self.create_line(x-ms,y-ms,x+ms,y+ms,fill=c)
                    self.create_line(x-ms,y+ms,x+ms,y-ms,fill=c)
                elif l == "fillsquare":
                    self.create_rectangle(x-ms,y-ms,x+ms,y+ms,fill=c,outline=c)

# FIXME:  No regression tester
class InteractiveGrapher(LabeledGrapher):
    def __init__(self,parent=None,**kw):
        LabeledGrapher.__init__(self,parent,**kw)    

    def unzoom(self):
        self.computeXRange()
        self.computeYRange()
        self.clear()
        self.draw()
        
    def labelPointWrapper(self,e):
        #find all objects near the mouse pointer
        points = self.find_closest(e.x,e.y)
        #find the first one that has a data_point?? tag
        point=None
        curve=None
        for point in points:
            #tags associated with the point
            point_tags = self.gettags(point)
            for tag in point_tags:
                tag=tag.split(":")
                if tag[0] == "data_point":
                    point=int(tag[1])
                if tag[0] == "curve":
                    curve=int(tag[1])
        if not((point is None) or (curve is None)):
            label = tkSimpleDialog.askstring("Label","Enter here")
            self.addLabel(curve,point,label)

    def printTagWrapper(self,e):
        theid=self.find("closest",e.x,e.y)
        print(self.gettags(theid[0]))
        
    def printValueWrapper(self,e):
        self.__printValue((e.x,e.y))

    def __printValue(self,val):
        print(self.canvasToValue(val))

    def zoomRubberBand(self,e):
        self.delete("rubber_band")
        self.create_polygon(e.x,e.y,e.x,self.zoom_starty,
                            self.zoom_startx,self.zoom_starty,
                            self.zoom_startx,e.y,
                            tags=("rubber_band",),fill="",outline="grey")

    def zoomWrapperStart(self,e):
        self.zoom_startx = e.x
        self.zoom_starty = e.y
        
    def zoomWrapperEnd(self,e):
        tolerance = 5

        cx=[self.zoom_startx,e.x]
        cy=[self.zoom_starty,e.y]
        cx.sort()
        cy.sort()
        
        if cx[1] - cx[0] > tolerance and cy[1] - cy[0] > tolerance:
            vx=[0,0]
            vy=[0,0]
            # canvas coordinates start at the top left, values coordinates start at bottom left
            vx[0],vy[0] = self.canvasToValue((cx[0],cy[1]))
            vx[1],vy[1] = self.canvasToValue((cx[1],cy[0]))

            self.computeXRange(vx[0],vx[1])
            self.computeYRange(vy[0],vy[1])
            self.clear()
            self.draw()
#            self.config(minx=vx[0],maxx=vx[1],miny=vy[0],maxy=vy[1])

    def drawWrapper(self,e):
        # Note: we should not update self["width"] and self["height"] here, because
        # those can be 2 or 4 less
        self._configNoDraw(realwidth=e.width,realheight=e.height)
        self.configure()
        self.clear()
        self.draw()

class GUIGrapher(InteractiveGrapher):
    def __init__(self,parent=None,**kw):
        InteractiveGrapher.__init__(self,parent,**kw)
        if not hasattr(self, "tk"):
            return
        self.bind("<ButtonPress-3>",self.popupMenuWrapper)
        self.menu=Tkinter.Menu()
        self.menu.add_radiobutton(label="print value",command=self.printValueBindings)
#        self.menu.add_radiobutton(label="print tag",command=self.printTagBindings)
#        self.menu.add_radiobutton(label="label point",command=self.labelPointBindings)
        self.menu.add_radiobutton(label="zoom",command=self.zoomBindings)
        self.menu.invoke('zoom')
        self.menu.add_command(label="Unzoom",command=self.unzoom)
        self.menu.add_command(label="Postscript",command=self.generatePostscript)
        self.menu.add_command(label="Configure...",command=self.__interactiveConfigureDialog)
        self.bind("<Configure>",self.drawWrapper)

    def __interactiveConfigureDialog(self):
        diag = Pmw.Dialog(self,buttons=("Ok","Cancel"))
        options = sorted([self.configure(key)[0] for key in self.configure() 
                          if self._isInternalOption(key)])

        self.optionList = Pmw.ScrolledListBox(diag.interior(),
                                              items=options,
                                              dblclickcommand=self.__updateInteractiveConfigureDialog)
        self.optionList.pack(side="left")

        frame = Tkinter.Frame(diag.interior())
        frame.pack(side="right")
        self.optionLabel = Pmw.EntryField(frame,
                                         labelpos="w",
                                         label_text="Option Name",
                                         entry_state=Tkinter.DISABLED)
        self.optionLabel.pack(side="top")

        self.valueLabel = Pmw.EntryField(frame,
                                         labelpos="w",
                                         label_text="Old Value",
                                         entry_state=Tkinter.DISABLED)
        self.valueLabel.pack(side="top")

        self.valueEntry = Pmw.EntryField(frame,
                                         labelpos="w",
                                         label_text="New Value",
                                         command=self.__modifyOption)
        self.valueEntry.pack(side="top")
        
    def __modifyOption(self):
        key = self.optionList.getcurselection()[0]
        if isinstance(self.cget(key), int):
            self[key] = int(self.valueEntry.get())
        elif isinstance(self.cget(key), float):
            self[key] = float(self.valueEntry.get())
        elif isinstance(self.cget(key), str):
            self[key] = self.valueEntry.get()
        self.valueLabel.setentry(self.cget(key))

    def __updateInteractiveConfigureDialog(self):
        key = self.optionList.getcurselection()[0]
        self.optionLabel.setentry(key)
        self.valueLabel.setentry(self.cget(key))
        self.valueEntry.clear()
        if isinstance(self.cget(key), int):
            self.valueEntry.configure(validate={"validator":"integer"})
        elif isinstance(self.cget(key), float):
            self.valueEntry.configure(validate={"validator":"real"})
        elif isinstance(self.cget(key), str):
            self.valueEntry.configure(validate={"validator":"alphanumeric"})
        
    def generatePostscript(self,filename=None,pscolormode=None):
        """
        Save the plot as postscript.
        """
        if pscolormode is None:
            pscolormode=self.cget("ps_colormode")
        if filename is None:
            filename = tkFileDialog.asksaveasfilename(defaultextension=".eps",title="Save as Postscript File")
        self.update()
        self.postscript(file=filename,colormode=pscolormode)

    savefig = generatePostscript

    def printValueBindings(self):
        self.bind("<ButtonPress-1>",self.printValueWrapper)
        self.unbind("<B1-Motion>")
        self.unbind("<ButtonRelease-1>")
        
    def printTagBindings(self):
        self.bind("<ButtonPress-1>",self.printTagWrapper)
        self.unbind("<B1-Motion>")
        self.unbind("<ButtonRelease-1>")
        
    def labelPointBindings(self):
        self.bind("<ButtonPress-1>",self.labelPointWrapper)
        self.unbind("<B1-Motion>")
        self.unbind("<ButtonRelease-1>")
        
    def zoomBindings(self):
        self.bind("<ButtonPress-1>",self.zoomWrapperStart)
        self.bind("<B1-Motion>",self.zoomRubberBand)
        self.bind("<ButtonRelease-1>",self.zoomWrapperEnd)
        
    def popupMenuWrapper(self,e):
        self.menu.tk_popup(e.x+self.winfo_rootx(),e.y+self.winfo_rooty())


def test(grapher=None):
    if grapher is None:
        grapher = GUIGrapher()
    data = [float(i)*0.1 for i in range(62)]

    grapher.addArray((data,list(map(math.sin,data))))
    grapher.addArray((data,list(map(math.cos,data))))
    grapher.addLabel(0,10,"hello")
    grapher.addLabel(0,30,"world")
    grapher.pack()
    grapher.plot()

    button = Tkinter.Button(text="Quit",command=grapher.quit)
    button.pack()
    button.update()
    print("Press <return> to continue")
    raw_input()

    grapher.delAllData()
    grapher.addArray((data,list(map(math.cos,data))))
    grapher.plot()
    print("Press <return> to continue")
    raw_input()

if __name__=='__main__':
    test()









#!/usr/bin/env python

#This is Randy Paffenroths attempt to make a better regression
#tester for AUTO.
import getopt
import os
import sys, shutil
import time
import platform
from auto.interactiveBindings import AUTOInteractiveConsole
from auto import AUTOclui
import parse_test
from auto import runDemo

# selection
DIR1=["pp2","exp","int","dd2","opt","lin","bvp","pp3","wav","plp"]

# HomCont selection
DIR2=["san","mtn","kpr","cir","she","rev"] 

# All AUTO97 demos
DIR3=DIR1+DIR2+["ab","abc","brc","brf","bru","chu","enz","ext","ezp","frc",
                "fsh","ivp","kar","lor","lrz","nag","non","obv","ops","pd1",
                "pd2","pen","phs","ppp","pvl","spb","stw","tim","tor"]
DIR3.sort()

# Plus new demos
DIR4=DIR3+["abcb","apbp","c2c","cusp","ffn","fhh","fhn","fnb","fnc","hen",
           "kdv","lcbp","log","man","nep","p2c","pcl","ph1","pla","python",
           "python/n-body","r3b","sib",
           "snh","sspg","tfc","um2","um3","vhb"]
DIR4.sort()

# dummy stdin to redirect wait()s
class nostdin(object):
    def read(self):
        return "\n"
    def readline(self):
        return "\n"

def test(demos, versions=None, log_file=None, parse=True):
    sys.stdin = nostdin()
    if demos == "selec":
        demos = DIR1
    if demos == "hom":
        demos = DIR2
    if demos == "all97":
        demos = DIR3
    if demos == "all":
        demos = DIR4

    if log_file is None:
        hostname = platform.node()
        if '.' in hostname:
            hostname = hostname.split('.')[0]
        log_file = hostname+'_log'

    if versions is None:
        versions = ['07p']

    log_files = []
    log = {}
    err = {}
    runner = {}
    for version in versions:
        if len(log_file)!=0:
            log_files.append(log_file+version)
            log[version] = open(log_file+version,"w")
            err[version] = open(log_file+version+"errors","w")
        else:
            log[version] = None
            err[version] = None
        runner[version] = AUTOInteractiveConsole(AUTOclui.exportFunctions(
                log[version],err[version]))

    autofiles = []
    for d in demos:
        print("Doing "+d)
        for version in versions:
            print("Version "+version)
            auto_dir=os.environ["AUTO_DIR"]
            demo_dir=os.path.join(auto_dir,"demos",d)
            autofiles = []
            if os.path.exists(os.path.join(demo_dir,"%s.auto"%d)):
                autofiles = ["%s.auto"%d]
            else:
                autofiles = [dirname for dirname in os.listdir(demo_dir) if
                             dirname[-5:]=='.auto']
                autofiles.sort()
            if len(autofiles) > 0:
                oldcwd = os.getcwd()
                tmpdir = os.path.join(oldcwd,'tmp')
                try:
                    shutil.rmtree(tmpdir)
                except OSError:
                    pass
                try:
                    os.mkdir(tmpdir)
                except OSError:
                    pass
                os.chdir(tmpdir)
                AUTOclui.copydemo(d)
                log[version].write("Demo %s is started\n"%d)
                for autofile in autofiles:
                    os.chdir(tmpdir)
                    runner[version].auto(autofile)
                os.chdir(oldcwd)
                log[version].write("Demo %s is done\n"%d)
            else:
                runDemo.runDemo(d, verbose="yes", log=log[version],
                                err=err[version], auto_dir=auto_dir)
    if len(autofiles) > 0:
        try:
            shutil.rmtree(tmpdir)
        except OSError:
            pass

    if len(log_file)!=0:
        for version in versions:
            runner[version].close()
            err[version].close()
            log[version].close()

    retcode = 0
    if parse:
        retcode = parse_test.parse(log_files, demos=demos)
    return retcode

if __name__ == '__main__':
    opts_list,args=getopt.getopt(sys.argv[1:],"cfpl:i:")

    #defaults
    versions = []
    log_file = ""

    opts={}
    for x in opts_list:
        opts[x[0]]=x[1]

    if "-c" in opts:
        versions.append("2000")
    if "-f" in opts:
        versions.append("97")
    if "-p" in opts:
        versions.append("07p")
    if "-l" in opts:
        log_file=opts["-l"]

    demos = args[0]
    sys.exit(test(args[0], versions, log_file, parse=False))



#!/usr/bin/env python
import matplotlib
matplotlib.use('TkAgg')

from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
try: #MPL 2.2 wants NavigationToolbar2Tk instead
    from matplotlib.backends.backend_tkagg import NavigationToolbar2Tk as NavigationToolbar2TkAgg
except ImportError:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2TkAgg
from matplotlib.backends.backend_agg import FigureCanvasAgg
from matplotlib.figure import Figure
from matplotlib.lines import Line2D
from matplotlib.axes import Axes
from matplotlib.ticker import AutoLocator, FixedLocator
from auto import Points
if not Points.numpyimported:
    Points.importnumpy()
transpose = Points.N.transpose

try: # mpl >= 0.99
    from mpl_toolkits.mplot3d import Axes3D
    from mpl_toolkits.mplot3d.proj3d import proj_transform, inv_transform
except ImportError:
    try: # mpl < 0.98
        from matplotlib.axes3d import Axes3D, Axes3DI
        from matplotlib.proj3d import proj_transform, inv_transform
        # mpl 0.87.7 bug workaround and compat
        def autoscale_view(self, scalex=True, scaley=True):
            self.set_top_view()
        Axes3DI.autoscale_view = autoscale_view
        Axes3DI.set_xlim3d = Axes3DI.set_w_xlim
        Axes3DI.set_ylim3d = Axes3DI.set_w_ylim
        Axes3DI.set_zlim3d = Axes3DI.set_w_zlim
        Axes3DI.get_xlim3d = Axes3DI.get_w_xlim
        Axes3DI.get_ylim3d = Axes3DI.get_w_ylim
        Axes3DI.get_zlim3d = Axes3DI.get_w_zlim
    except (ImportError, NotImplementedError, NameError):
        Axes3D = None

try:
    import Tkinter
    import tkFileDialog
except ImportError:
    import tkinter as Tkinter # Python 3
    from tkinter import filedialog as tkFileDialog
from auto.graphics import Pmw
from auto import AUTOutil
import math
import string
from auto.graphics import grapher

GrapherError="GrapherError"

class FigureCanvasTkAggRedraw(FigureCanvasTkAgg):
    def __init__(self,grapher,parent):
        if parent is None:
            parent = Tkinter.Tk()
        parent.wm_title("PyPLAUT")
        FigureCanvasTkAgg.__init__(self,grapher.ax.get_figure(),master=parent)
        
        grapher.toolbar = NavigationToolbar2TkAgg( self, parent )
        grapher.toolbar.zoom()

        self.grapher = grapher

    def redraw(self):
        # recalculate label positions
        self.grapher.plotlabels()
        FigureCanvasTkAgg.draw(self)
        
    def draw(self):
        ax = self.grapher.ax
        d = {}
        if ax is self.grapher.ax3d:
            [d["minx"],d["maxx"]] = ax.get_xlim3d()
            [d["miny"],d["maxy"]] = ax.get_ylim3d()
            [d["minz"],d["maxz"]] = ax.get_zlim3d()
            d["azimuth"] = ax.azim
            d["elevation"] = ax.elev
            d["cur_lims"] = Axes.get_xlim(ax), Axes.get_ylim(ax)
        else:
            [d["minx"],d["maxx"]] = ax.get_xlim()
            [d["miny"],d["maxy"]] = ax.get_ylim()
        fig = ax.get_figure()
        dpi = fig.get_dpi()
        d["realwidth"] = fig.get_figwidth()*dpi
        d["realheight"] = fig.get_figheight()*dpi
        for k in list(d):
            # don't adjust any unchanged settings
            if k == "cur_lims":
                if map(list, d[k]) == map(list, self.grapher._cur_lims):
                    del d[k]
            elif d[k] == self.grapher.cget(k):
                del d[k]
        if d != {}:
            if "cur_lims" in d:
                del d["cur_lims"]
            if d != {}:
                self.grapher._configNoDraw(**d)
            self.redraw()
            return
        FigureCanvasTkAgg.draw(self)

    # MPL 2.2+ always calls draw() and never show() (deprecated)
    # Older MPLs call one or the other.
    def show(self):
        self.draw()

class BasicGrapher(grapher.BasicGrapher):
    """Documentation string for Basic Grapher

    A simple graphing widget
    By Randy P. and Bart O."""
    def __init__(self,parent=None,**kw):
        self.ax = Figure(figsize=(4.3,3.0)).gca()
        self.ax2d = self.ax
        self.ax3d = None
        if kw.get("hide"):
            self.canvas = FigureCanvasAgg(self.ax.get_figure())
        else:
            self.canvas = FigureCanvasTkAggRedraw(self,parent)
            tk_widget = self.canvas.get_tk_widget()
            self.quit = tk_widget.quit
            self.bind = tk_widget.bind
            self.unbind = tk_widget.unbind
            self.winfo_rootx = tk_widget.winfo_rootx
            self.winfo_rooty = tk_widget.winfo_rooty
        self.ax.set_autoscale_on(0)
        self.postscript = self.canvas.print_figure
        self.savefig = self.ax.get_figure().savefig

        self.redrawlabels = 0

        callback = self.__optionCallback
        grapher.BasicGrapher.__init__(self,parent,callback,**kw)
        optionDefaults={}
        optionDefaults["xticks"] = (None,callback)
        optionDefaults["yticks"] = (None,callback)
        optionDefaults["zticks"] = (None,callback)
        optionDefaults["background"] = ("white",callback)
        optionAliases = {}
        optionAliases["bg"] = "background"

        self.addOptions(**optionDefaults)
        self.addAliases(**optionAliases)

        for key in ["grid","decorations","xlabel","ylabel","zlabel",
                    "xlabel_fontsize", "ylabel_fontsize", "zlabel_fontsize",
                    "minx","maxx","miny","maxy","minz","maxz", "width",
                    "height", "left_margin", "right_margin", "top_margin",
                    "bottom_margin"]:
            self.__optionCallback(key,self.cget(key),[])
        matplotlib.rcParams["axes.edgecolor"]=self.cget("foreground")

    def pack(self,**kw):
        self.canvas.get_tk_widget().pack(kw)

    def update(self):
        if isinstance(self.canvas,FigureCanvasTkAggRedraw):
            self.canvas.get_tk_widget().update()
            FigureCanvasTkAgg.draw(self.canvas)
        else:
            self.canvas.draw()

    def __optionCallback(self,key,value,options):
        if key in ["minx","maxx","miny","maxy","minz","maxz",
                   "realwidth","realheight","azimuth","elevation",
                   "left_margin","right_margin","top_margin","bottom_margin"]:
            self.redrawlabels = 1
            if key[:3] in ["min", "max"]:
                minc = self.cget("min"+key[3])
                maxc = self.cget("max"+key[3])
                if minc < maxc:
                    func = None
                    if self.ax is self.ax3d:
                        func = getattr(self.ax, "set_"+key[3]+"lim3d")
                        self._cur_lims = (
                            Axes.get_xlim(self.ax), Axes.get_ylim(self.ax))
                    elif key[3] != 'z':
                        func = getattr(self.ax, "set_"+key[3]+"lim")
                    if func is not None:
                        func(minc,maxc)
                        tickskey = key[3]+"ticks"
                        ticksval = self.cget(tickskey)
                        if ticksval is not None:
                            self.__optionCallback(tickskey,ticksval,options)
            elif key == "realwidth":
                lm = float(self.cget("left_margin"))
                rm = float(self.cget("right_margin"))
                self.ax.get_figure().subplots_adjust(left=lm/value,
                                                     right=1-rm/value)
            elif key == "realheight":
                tm = float(self.cget("top_margin"))
                bm = float(self.cget("bottom_margin"))
                top = 1-tm/value
                bottom = bm/value
                if top > bottom:
                    self.ax.get_figure().subplots_adjust(top=top, bottom=bottom)
            elif key == "left_margin":
                fig = self.ax.get_figure()
                width = fig.get_figwidth()*fig.get_dpi()
                fig.subplots_adjust(left=value/width)
            elif key == "right_margin":
                fig = self.ax.get_figure()
                width = fig.get_figwidth()*fig.get_dpi()
                fig.subplots_adjust(right=1-value/width)
            elif key == "top_margin":
                fig = self.ax.get_figure()
                height = fig.get_figheight()*fig.get_dpi()
                fig.subplots_adjust(top=1-value/height)
            elif key == "bottom_margin":
                fig = self.ax.get_figure()
                height = fig.get_figheight()*fig.get_dpi()
                fig.subplots_adjust(bottom=value/height)
            elif self.ax is self.ax3d:
                elev = self.cget("elevation")
                azim = self.cget("azimuth")
                if elev is not None or azim is not None:
                    self.ax.view_init(elev, azim)
        elif key == "grid":
            if value in ["yes", True]:
                self.ax.grid(color = self.cget("foreground"))
            else:
                self.ax.grid(False)
        elif key in ["width", "height"]:
            if isinstance(self.canvas,FigureCanvasTkAggRedraw):
                self.canvas.get_tk_widget()[key] = value
            else:
                fig = self.ax.get_figure()
                if key == "width":
                    fig.set_figwidth(float(value)/fig.get_dpi())
                    self._configNoDraw(realwidth=value)
                else:
                    fig.set_figheight(float(value)/fig.get_dpi())
                    self._configNoDraw(realheight=value)
        elif key == "top_title":
            fontsize = self.cget("top_title_fontsize")
            if fontsize is None:
                self.ax.set_title(value)
            else:
                self.ax.set_title(value,fontsize=fontsize)
        elif key == "top_title_fontsize":
            title = self.cget("top_title")
            if title is not None:
                self.ax.set_title(title,fontsize=value)
        elif key in ["background", "bg"]:
            if hasattr(self.ax, 'set_facecolor'):
                # matplotlib >= 2.0
                self.ax.set_facecolor(value)
            else:
                self.ax.set_axis_bgcolor(value)
        elif key in ["foreground", "fg"]:
            matplotlib.rcParams["axes.edgecolor"]=self.cget("foreground")
            self.redrawlabels = 1
            if self.cget("grid") in ["yes", True]:
                self.ax.grid(color=value)
        elif key == "color_list":
            color_list = value.split()
            i = 0
            for d in self.data:
                if d["newsect"] is None or d["newsect"]:
                    i = i+1
                if d["color"] is None:
                    color = i
                else:
                    color = d["color"]
                d["mpline"].set_color(color_list[color%len(color_list)])
        elif key == "decorations":
            if value:
                self.ax.set_axis_on()
            else:
                self.ax.set_axis_off()
        elif key == "use_symbols":
            self.plotsymbols()
        elif key == "use_labels":
            self.plotlabels()
        elif key in ["xlabel","ylabel","zlabel"]:
            if value is None:
                value = ""
            fontsize = self.cget(key+"_fontsize")
            if hasattr(self.ax,"set_"+key):
                func = getattr(self.ax,"set_"+key)
                if fontsize is None:
                    func(value)
                else:
                    func(value,fontsize=fontsize)
        elif key in ["xlabel_fontsize","ylabel_fontsize","zlabel_fontsize"]:
            label = self.cget(key[:6])
            if hasattr(self.ax,"set_"+key[:6]):
                func = getattr(self.ax,"set_"+key[:6])
                if value is None:
                    func(label)
                else:
                    func(label,fontsize=value)
        elif key in ["xticks","yticks","zticks"]:
            if value is None:
                if self.ax is self.ax3d:
                    axis = getattr(self.ax, "w_"+key[0]+"axis")
                    axis.set_major_locator(AutoLocator())
                elif key == "xticks":
                    self.ax.set_xscale('linear')
                else:
                    self.ax.set_yscale('linear')
            else:
                min = self.cget("min"+key[0])
                max = self.cget("max"+key[0])
                ticks = [min + ((max - min) * i) / float(value-1)
                         for i in range(value)]
                if self.ax is self.ax3d:
                    axis = getattr(self.ax, "w_"+key[0]+"axis")
                    axis.set_major_locator( FixedLocator(ticks) )
                elif key == "xticks":
                    self.ax.set_xticks(ticks)
                elif key == "yticks":
                    self.ax.set_yticks(ticks)

    def _delAllData(self):
        for d in self.data:
            if "mpline" in d:
                self.ax.lines.remove(d["mpline"])
        self.data=[]

        # set type for next data
        try:
            zcolumn = self.cget(self.cget("type")+"_z")
        except Tkinter.TclError: #in regression test
            return
        oldax = self.ax
        if zcolumn is None or Axes3D is None:
            if zcolumn is not None:
                self._configNoDraw({self.cget("type")+"_z":None})
                print("\nmatplotlib 0.98.x does not support 3D plots.")
                print("Plotting only the first two coordinates.")
            if self.ax is self.ax3d:
                # restore zoom mode
                new_zoom_mode = self.zoom_mode
                self.ax = self.ax2d
        else:
            if self.ax3d is None:
                self.ax3d = Axes3D(self.ax.get_figure())
                try:
                    self.ax3d.set_autoscale_on(0)
                except TypeError: #Matplotlib 1.1 bug
                    self.ax2d.set_autoscale_on(0)
                    self.ax3d.set_autoscalez_on(0)
            if self.ax is self.ax2d:
                # remember zoom mode and disable zoom
                if isinstance(self.canvas,FigureCanvasTkAggRedraw):
                    self.zoom_mode = self.toolbar.mode
                    new_zoom_mode = ''
                self.ax = self.ax3d
        if self.ax is not oldax:
            if (isinstance(self.canvas,FigureCanvasTkAggRedraw) and
                new_zoom_mode != self.toolbar.mode):
                if "rect" in new_zoom_mode:
                    self.toolbar.zoom()
                elif "pan" in new_zoom_mode:
                    self.toolbar.pan()
                elif "rect" in self.toolbar.mode:
                    self.toolbar.zoom()
                elif "pan" in self.toolbar.mode:
                    self.toolbar.pan()
            #copy settings from 3d to 2d or vice versa
            for key in ("grid","decorations","xlabel","ylabel","zlabel",
                        "xticks","yticks","zticks",
                        "minx","maxx","miny","maxy","minz","maxz",
                        "azimuth", "elevation",
                        "top_title", "background"):
                self.__optionCallback(key,self.cget(key),[])

    def _delData(self,index):
        if "mpline" in data[index]:
            self.ax.lines.remove(data[index]["mpline"])
        del self.data[index]

    def clear(self):
        if len(self.ax.get_figure().axes) > 0:
            self.ax.get_figure().delaxes(self.ax.get_figure().axes[0])

    def draw(self):
        if self.redrawlabels:
            self.plotlabels()
        if len(self.ax.get_figure().axes) == 0:
            self.ax.get_figure().add_axes(self.ax)
        if isinstance(self.canvas,FigureCanvasTkAggRedraw):
            FigureCanvasTkAgg.draw(self.canvas)
        else:
            self.canvas.draw()

    def plot(self):
        color_list = self.cget("color_list").split()
            
        # data
        line_width=self.cget("line_width")
        dashes=list(map(float,self.cget("dashes")))
        i=-1
        for d in self.data:
            if d["newsect"] is None or d["newsect"]:
                i = i+1
            curve="curve:%d"%(i,)
            if self.ax is self.ax2d:
                v = [d["x"],d["y"]]
            else:
                v = [d["x"],d["y"],d["z"]]
            if d["color"] is None:
                color = i
            else:
                color = d["color"]
            kw = {'color':color_list[color%len(color_list)]}
            if len(v[0]) == 1:
                # If we only have one point we draw a small circle or a pixel
                if self.cget("type") == "solution":
                    marker = 'o'
                else:
                    marker = ','
                v.append(marker)
                #tags=("data_point:%d"%(0,),curve,"data")
            else:
                stable = d["stable"]
                #tags=(curve,"data")
                kw['lw'] = line_width
                if stable is not None and not stable:
                    kw.update({'ls':'--','dashes':dashes})
            if self.ax is self.ax2d:
                self.ax.plot(*v,**kw)
            else:
                self.ax.plot3D(*v,**kw)
            d["mpline"] = self.ax.lines[-1]
        if len(self.ax.get_figure().axes) == 0:
            self.ax.get_figure().add_axes(self.ax)
            
    def __setitem__(self,key,value):
        self.configure(**{key: value})

    def __getitem__(self,key):
        return self.cget(key)

class LabeledGrapher(BasicGrapher,grapher.LabeledGrapher):
    def __init__(self,parent=None,**kw):
        self.labels=[]
        BasicGrapher.__init__(self,parent,**kw)

    def _delData(self,i):
        del self.labels[i]
        BasicGrapher._delData(self,i)

    def _delAllData(self):
        for l in self.labels:
            for label in l:
                if "mpline" in label:
                    self.ax.lines.remove(label["mpline"])
                if "mptext" in label:
                    self.ax.texts.remove(label["mptext"])
                if "mpsymline" in label:
                    self.ax.lines.remove(label["mpsymline"])
                if "mpsymtext" in label:
                    self.ax.texts.remove(label["mpsymtext"])
        self.labels=[]
        BasicGrapher._delAllData(self)

    def _addData(self,data,newsect=None,color=None,stable=None):
        self.labels.append([])
        BasicGrapher._addData(self,data,newsect,color,stable)

    def plotlabels(self):
        if self.cget("realwidth") == 1 or self.cget("realheight") == 1:
            return
        self.redrawlabels = 0

        for labels in self.labels:
            for label in labels:
                if "mpline" in label:
                    self.ax.lines.remove(label["mpline"])
                    del label["mpline"]
                if "mptext" in label:
                    self.ax.texts.remove(label["mptext"])
                    del label["mptext"]

        if not self.cget("use_labels"):
            return

        if hasattr(self.ax.transData,'transform'):
            trans = self.ax.transData.transform
            if self.ax is self.ax2d:
                def transseq(x,y):
                    return trans(transpose([x,y]))
            else:
                # this matrix needs to be initialized or matplotlib only does
                # that when it draws
                self.ax.M = self.ax.get_proj()
                def transseq(x,y,z):
                    return trans(transpose(proj_transform(x,y,z,self.ax.M)[:2]))
            inv_trans = self.ax.transData.inverted().transform
        else:
            trans = self.ax.transData.xy_tup
            if self.ax is self.ax2d:
                def transseq(x,y):
                    return transpose(self.ax.transData.numerix_x_y(x,y))
            else:
                self.ax.M = self.ax.get_proj()
                def transseq(x,y,z):
                    return transpose(self.ax.transData.numerix_x_y(
                            *(proj_transform(x,y,z,self.ax.M)[:2])))
            inv_trans = self.ax.transData.inverse_xy_tup
        if self.cget("smart_label"):
            mp = self.inarrs()
            sp1 = self.cget("left_margin")
            sp2 = 5 #fontsize
            sp3 = self.cget("bottom_margin")
            sp4 = 5
            for d in self.data:
                if self.ax is self.ax2d:
                    seq = transseq(d["x"],d["y"])
                else:
                    seq = transseq(d["x"],d["y"],d["z"])
                seq[:,0] = (seq[:,0] - sp1) / sp2
                seq[:,1] = (seq[:,1] - sp3) / sp4
                self.map_curve(mp,seq)
                                                      
        for i, labels in enumerate(self.labels):
            for label in labels:
                if len(label["text"]) == 0:
                    continue
                [x,y] = label["xy"][:2]
                if self.ax is self.ax3d:
                    # transform 3d to 2d coordinates and compare
                    # against the 2D limits
                    z = label["xy"][2]
                    [x,y,z] = proj_transform(x, y, z, self.ax.M)
                lim = Axes.get_xlim(self.ax)
                if x < lim[0] or x > lim[1]:
                    continue
                lim = Axes.get_ylim(self.ax)
                if y < lim[0] or y > lim[1]:
                    continue
                data = trans((x,y))
                if data is not None:
                    [x,y] = data
                    if self.cget("smart_label"):
                        [xoffd1,yoffd1,xoffd2,yoffd2,
                         xofft,yofft,pos] = self.findsp(x,y,mp)
                    else:
                        [xoffd1,yoffd1,xoffd2,yoffd2,
                         xofft,yofft,pos] = self.dumblabel(i,label["j"],x,y)
                    [ha,va] = self.getpos(pos)
                    [xd1,yd1] = inv_trans((x+xoffd1,y+yoffd1))
                    [xd2,yd2] = inv_trans((x+xoffd2,y+yoffd2))
                    [xt,yt] = inv_trans((x+xofft,y+yofft))
                    # using the low-level line routines to avoid
                    # rescaling in 3D (even with auto scale off,
                    # the view is reset after each plot() by
                    # set_top_view in mplot3d)
                    line = Line2D([xd1,xd2],[yd1,yd2],linewidth=0.5,
                                  color=self.cget("foreground"))
                    self.ax.add_line(line)
                    self.ax.annotate(label["text"],(xt,yt),ha=ha,va=va,
                                     color=self.cget("foreground"),clip_on=True)
                    label["mpline"] = self.ax.lines[-1]
                    label["mptext"] = self.ax.texts[-1]

    def getpos(self,pos):
        has = [  "left", "center", "right", "right", "right", "center",
                 "left", "left", "left" ]
        vas = [  "bottom","bottom","bottom","center", "top", "top",
                 "top", "center", "bottom" ]
        ha = has[pos]
        va = vas[pos]
        return [ha,va]

    def plot(self):
        self.plotlabels()
        BasicGrapher.plot(self)
        self.plotsymbols()

    def plotsymbols(self):
        for labels in self.labels:
            for label in labels:
                v = label["xy"]
                if Axes3D is None:
                    v = v[:2]
                l = label["symbol"]
                if l is None:
                    continue
                if "mpsymline" in label:
                    self.ax.lines.remove(label["mpsymline"])
                    del label["mpsymline"]
                if "mpsymtext" in label:
                    self.ax.texts.remove(label["mpsymtext"])
                    del label["mpsymtext"]
                if not self.cget("use_symbols"):
                    continue
                c=self.cget("symbol_color")
                if len(l) <= 3:
                    #font=self.cget("symbol_font"),
                    kw = {'ha':"center",'va':"center",'color':c,'clip_on':True}
                    if self.ax is self.ax2d:
                        self.ax.text(*(v+[l]),**kw)
                    else:
                        t = self.ax.text3D(*(v+[l]))
                        t.set_horizontalalignment(kw['ha'])
                        t.set_verticalalignment(kw['va'])
                        t.set_color(kw['color'])
                        t.set_clip_on(True)
                    label["mpsymtext"] = self.ax.texts[-1]
                    continue
                v = [[coord] for coord in v]
                markerdict = {"fillcircle" : "o", "circle": "o",
                              "square": "s", "crossquare": "x",
                              "fillsquare": "s", "diamond": "D",
                              "filldiamond": "D", "triangle": "^",
                              "doubletriangle": "^"}
                ms = None
                for i, ch in enumerate(l):
                    if ch in string.digits:
                        ms = int(l[i:])
                        l = l[:i]
                        break
                if l not in markerdict:
                    continue
                v.append(markerdict[l])
                kw = {'color': c}
                if l in ["circle", "square", "diamond", "triangle"]:
                    if hasattr(self.ax, 'get_facecolor'):
                        # matplotlib >= 2.0
                        kw['mfc'] = self.ax.get_facecolor()
                    else:
                        kw['mfc'] = self.ax.get_axis_bgcolor()
                if l in ["diamond","filldiamond","triangle","doubletriangle"]:
                    kw['ms'] = 8
                if ms is not None:
                    kw['ms'] = ms
                if self.ax is self.ax2d:
                    self.ax.plot(*v,**kw)
                else:
                    self.ax.plot3D(*v,**kw)
                label["mpsymline"] = self.ax.lines[-1]

# FIXME:  No regression tester
class InteractiveGrapher(LabeledGrapher,grapher.InteractiveGrapher):
    def __init__(self,parent=None,**kw):
        LabeledGrapher.__init__(self,parent,**kw)

class GUIGrapher(InteractiveGrapher,grapher.GUIGrapher):
    def __init__(self,parent=None,**kw):
        InteractiveGrapher.__init__(self,parent,**kw)
        #self.bind("<ButtonPress-3>",self.popupMenuWrapper)
        if kw.get("hide"):
            return
        self.menu=Tkinter.Menu()
#        self.menu.add_radiobutton(label="print tag",command=self.printTagBindings)
#        self.menu.add_radiobutton(label="label point",command=self.labelPointBindings)
        #self.menu.add_radiobutton(label="zoom",command=self.zoomBindings)
        #self.menu.invoke('zoom')
        self.menu.add_command(label="Unzoom",command=self.unzoom)
        self.menu.add_command(label="Save",command=self.generatePostscript)
        self.menu.add_command(label="Configure...",command=self.__interactiveConfigureDialog)

    def __interactiveConfigureDialog(self):
        widget=self.canvas.get_tk_widget()
        diag = Pmw.Dialog(widget,buttons=("Ok","Cancel"))
        options = sorted([self.configure(key)[0] for key in self.configure() 
                          if self._isInternalOption(key)])

        self.optionList = Pmw.ScrolledListBox(diag.interior(),
                                              items=options,
                                              dblclickcommand=self.__updateInteractiveConfigureDialog)
        self.optionList.pack(side="left")

        frame = Tkinter.Frame(diag.interior())
        frame.pack(side="right")
        self.optionLabel = Pmw.EntryField(frame,
                                         labelpos="w",
                                         label_text="Option Name",
                                         entry_state=Tkinter.DISABLED)
        self.optionLabel.pack(side="top")

        self.valueLabel = Pmw.EntryField(frame,
                                         labelpos="w",
                                         label_text="Old Value",
                                         entry_state=Tkinter.DISABLED)
        self.valueLabel.pack(side="top")

        self.valueEntry = Pmw.EntryField(frame,
                                         labelpos="w",
                                         label_text="New Value",
                                         command=self.__modifyOption)
        self.valueEntry.pack(side="top")
        
    def generatePostscript(self,filename=None,pscolormode=None):
        if pscolormode is None:
            pscolormode=self.cget("ps_colormode")
        if filename is None:
            filename = tkFileDialog.asksaveasfilename(defaultextension=".eps",title="Save the figure")
        self.update()
        #self.postscript(filename,colormode=pscolormode)
        self.postscript(filename)

def test():
    grapher.test(GUIGrapher())

if __name__=='__main__':
    test()









import os

api_rst_path = os.path.join("source/api/pyfurc.rst")
with open(api_rst_path) as inf:
    lines = inf.readlines()

with open(api_rst_path, "w") as outf:
    i_line = 0
    while i_line < len(lines):
        line = lines[i_line]
        if "pyfurc package" in line:
            heading = "pyfurc Reference"
            introduction = """\
                This is the reference page for classes and methods in the
                pyfurc package. It is automatically rendered from
                docstrings in the source code.
                """
            outl = heading + "\n" + len(heading) * "=" + "\n"
            outl += introduction + "\n"
            i_line += 3
        elif "Submodules" in line:
            outl = ""
            i_line += 2
        else:
            outl = line
            i_line += 1
        outf.write(outl)
